package com.aleax.voip;

import java.net.DatagramSocket;
import java.net.SocketException;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.aleax.voip.ui.AppConfig;
import com.aleax.voip.util.LogUtil;

/**
 * Manages VOIP calls.
 * 
 * @author Yiannis Paschalidis
 */
public final class CallManager
{
   /** The logger instance for this class. */
   private static final Logger log = LogUtil.get(CallManager.class);
   
   /** Receives Audio data from the connected Peer and sends it to the audio output device. */
   private static ReceiveAudioThread receiveThread = new ReceiveAudioThread();
   
   /** Receives Audio data from the audio input device and sends it to the connected Peer. */
   private static SendAudioThread sendThread = new SendAudioThread();
   
   /** The currently connected Peer. */
   private static Peer peer;
   
   /** The socket used to transport audio data. */
   private static DatagramSocket socket;
   
   static
   {
      try
      {
         socket = new DatagramSocket(AppConfig.getUdpPort());
         receiveThread.setSocket(socket);
         sendThread.setSocket(socket);
      }
      catch (SocketException e)
      {
         // TODO Handle exception properly
         log.log(Level.WARNING, "Failed to create VOIP UDP socket - calls unavailable", e);
      }
      
      receiveThread.start();
      sendThread.start();
   }
   
   /** Prevent instantiation of this utility class. */
   private CallManager()
   {
   }

   /**
    * Sets the active VOIP call.
    * @param peer the Peer to exchange audio data with. 
    */
   public static void setActiveCall(final Peer peer)
   {
      setPeer(peer);
   }

   /**
    * Indicates whether there is currently a VOIP call in progress.
    * @return true if there is a VOIP call in progess, false otherwise.
    */
   public static boolean isCallActive()
   {
      synchronized (CallManager.class)
      {
         return peer != null;
      }
   }
   
   /** @return the currently connected peer, or null if there is no peer connected.. */
   public static Peer getActivePeer()
   {
      synchronized (CallManager.class)
      {
         return peer;
      }
   }

   /**
    * Terminates the current call (if any).
    */
   public static void endCall()
   {
      setPeer(null);
   }
   
   /**
    * Sets the currently connected Peer.
    * @param peer the peer to set.
    */
   private static void setPeer(final Peer peer)
   {
      synchronized (CallManager.class)
      {
         CallManager.peer = peer;
         
         receiveThread.setPeer(peer);      
         sendThread.setPeer(peer);
         
         sendThread.setEncrypted(AppConfig.isEncrypted());
      }
   }   
}
