package com.aleax.voip;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.Socket;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.sound.sampled.AudioFormat;

import com.aleax.voip.ui.AppConfig;
import com.aleax.voip.util.AudioUtil;
import com.aleax.voip.util.LogUtil;
import com.aleax.voip.util.Util;

/**
 * Represents a Peer running the VoipApp on another PC.
 * 
 * @author Yiannis Paschalidis.
 */
public final class Peer
{
   /** Describes the the VOIP call state. */
   public enum CallState
   {
      /** No active call. */
      INACTIVE,
      /** We are calling this peer. */
      DIALLING,
      /** This peer is calling us. */
      RINGING,
      /** This peer is busy with another call. */
      BUSY,
      /** Call connected - voice comms active. */
      CONNECTED
   }
   
   /** The logger instance for this class. */
   private static final Logger log = LogUtil.get(Peer.class);
   
   /** Delimiter for separating fields in messages. */
   private static final char DELIMITER = '\u0007';

   /** Delimiter for separating fields in messages. */
   private static final String DELIMITER_STR = "\u0007";
   
   /** The character set used to exchange messages. */
   private static final String CHARSET = "UTF-8";
   
   /** Ping interval, in milliseconds. */
   private static final long PING_INTERVAL = 10000;
   
   /** Prefix for handshake. */
   private static final String HANDSHAKE = "!HELO ";
   
   /** Prefix for messages. */
   private static final String MESSAGE = "!MESG ";
   
   /** Prefix for establishing a voip call. */
   private static final String DIAL = "!DIAL ";

   /** Prefix for accepting a voip call. */
   private static final String PICKUP = "!PICKUP ";
   
   /** Message for call rejected due to another call in progress. */
   private static final String BUSY = "!BUSY";
   
   /** Message for rejecting/ending a voip call. */
   private static final String END_CALL = "!ENDCALL";
   
   /** Keep-alive ping request. */
   private static final String PING = "!PING"; // TODO: Can put status info like busy in call, typing etc. on this.
   
   /** Keep-alive ping response. */
   private static final String PONG = "!PONG";
   
   /** The Peer's IP address. */
   private final String ip;
   
   /** The Peer's nickname. */
   private String name;
 
   /** The TCP socket used to send/receive commands. */
   private final Socket socket;
   
   /** The writer connected to the {@link #socket}. */
   private final PrintWriter out;
   
   /** The reader connected to the {@link #socket}. */
   private final BufferedReader in;
   
   /** Thread which performs blocking reads on {@link #in}. */
   private final ReadThread readThread;

   /** Thread which periodically sends a ping request to the Peer to ensure that it is still connected. */
   private final PingThread pingThread;

   /** The UDP port to use for VOIP calls. */
   private int udpPort;
   
   /** True if traffic from this Peer is encrypted. */
   private boolean encrypted;
   
   /** The Peer audio format. */
   private AudioFormat audioFormat;
   
   /** Used to synchronize access to {@link #callState}. */
   private final Object CALL_LOCK = new Object();
   
   /** The Peer's call state. */
   private CallState callState = CallState.INACTIVE;
   
   /**
    * Attempts to create a Peer connection.
    * @param socket the socket to attempt to connect with.
    * @throws Exception if a connection could not be established.
    */
   public Peer(final Socket socket) throws Exception
   {
      this.socket = socket;
      
      try
      {
         in = new BufferedReader(new InputStreamReader(socket.getInputStream(), CHARSET));
         out = new PrintWriter(new OutputStreamWriter(socket.getOutputStream(), CHARSET));
         ip = socket.getInetAddress().getHostAddress();
         
         doHandshake();
         
         readThread = new ReadThread();
         readThread.start();
         
         pingThread = new PingThread();
         pingThread.start();
      }
      catch (Exception e)
      {
         disconnect();
         throw e;
      }
   }

   /**
    * Attempts to connect and negotiate connection settings.
    * @throws Exception if the handshake failed.
    */
   private void doHandshake() throws Exception
   {
      log.info("Connecting peer " + ip);
      
      send(HANDSHAKE + AppConfig.getNickname() 
                     + DELIMITER + AppConfig.getUdpPort()
                     + DELIMITER + AppConfig.isEncrypted());
      
      String handShake = read();
      
      if (handShake.startsWith(HANDSHAKE))
      {
         String[] peerConfig = handShake.substring(HANDSHAKE.length()).split(DELIMITER_STR); 
         name = peerConfig[0];
         udpPort = Integer.parseInt(peerConfig[1]);
         encrypted = "true".equals(peerConfig[2]);
      }
      
      if (name == null || name.length() < 1 || udpPort < 1 || udpPort > 65535)
      {
         throw new IllegalStateException("Invalid handshake");
      }      
   }
   
   /**
    * Sends a text message to the peer.
    * @param message the message to send.
    */
   public void sendMessage(final String message)
   {
      if (message != null && message.length() > 0)
      {
         // TODO: Correctly handle \r\n as a single line feed
         send(MESSAGE + message.replaceAll("[\r\n]", DELIMITER_STR));
      }
   }
   
   /**
    * Accepts an incoming VOIP call from this peer.
    */
   public void pickup()
   {
      synchronized (CALL_LOCK)
      {
         switch (callState)
         {
            case RINGING:
            {
               send(PICKUP + AudioUtil.encodeAudioFormat(AppConfig.getRecordingDevice().getSecond()));
               CallManager.setActiveCall(this);
               setCallState(CallState.CONNECTED);
               break;
            }
            
            default:
               throw new IllegalStateException("Can't pick up when: " + callState);
         }
      }
   }
   
   /**
    * Requests a VOIP call with this peer.
    */
   public void dial()
   {
      synchronized (CALL_LOCK)
      {
         switch (callState)
         {
            case INACTIVE:
            case BUSY:
            {
               send(DIAL + AudioUtil.encodeAudioFormat(AppConfig.getRecordingDevice().getSecond()));
               setCallState(CallState.DIALLING);
               break;
            }
            
            default:
               throw new IllegalStateException("Can't start call when: " + callState);
         }
      }
   }
   
   /**
    * Terminates a VOIP call or incoming/outgoing call request with this peer.
    */
   public void hangup()
   {
      synchronized (CALL_LOCK)
      {
         switch (callState)
         {
            case DIALLING:
            case RINGING:
            case CONNECTED:
            {
               send(END_CALL);
               CallManager.endCall();
               setCallState(CallState.INACTIVE);
               break;
            }
            
            default:
               throw new IllegalStateException("Can't end call when: " + callState);
         }
      }
   }
   
   /**
    * Sets the call state and notifies listeners of the state change.
    * @param newState the state to set.
    */
   private void setCallState(final CallState newState)
   {
      CallState oldState = callState;
      this.callState = newState;
      
      if (!Util.equals(oldState, newState))
      {
         for (PeerListener listener : Peers.getListeners())
         {
            listener.peerCallStatusChanged(Peer.this, oldState);
         }      
      }
   }

   /**
    * Sends the given command to the peer.
    * @param command the command to send.
    */
   private void send(final String command)
   {
      if (isConnected())
      {
         try
         {
            synchronized (out)
            {
               out.println(command);
               out.flush();
            }
         }
         catch (Exception e)
         {
            disconnect();
         }
      }
   }
   
   /**
    * Reads a command from the Peer.
    * No validation is performed on the command.
    * 
    * @return the read command.
    * @throws IOException if the read failed.
    */
   private String read() throws IOException
   {
      synchronized (in)
      {
         return in.readLine();
      }
   }
   
   /**
    * @return the nickname.
    */
   public String getName()
   {
      return name;
   }
   
   /** @return the Peer's IP address. */
   public String getIp()
   {
      return ip;
   }
   
   /**
    * Indicates whether the Peer is currently connected.
    * @return true if the Peer is connected, false otherwise.
    */
   public boolean isConnected()
   {
      return socket != null && socket.isConnected() && !socket.isClosed();
   }
   
   /**
    * Gracefully disconnects this Peer.
    */
   public void disconnect()
   {
      pingThread.interrupt();
      readThread.interrupt();
      
      log.info("Disconnecting peer " + ip);
      Util.safeClose(in);
      Util.safeClose(out);
      Util.safeClose(socket);
      Peers.removePeer(this);
      
      synchronized (CALL_LOCK)
      {
         if (callState == CallState.DIALLING
               || callState == CallState.RINGING
               || callState == CallState.CONNECTED)
         {
            CallManager.endCall();
            setCallState(CallState.INACTIVE);
         }
      }
   }
   
   /** @return the current call state. */
   public CallState getCallState()
   {
      return callState;
   }

   /** @return the UDP port used for VOIP calls. */
   public int getUdpPort()
   {
      return udpPort;
   }

   /** @return true if traffic from this peer is encrypted. */
   public boolean isEncrypted()
   {
      return encrypted;
   }
   
   /** @return the Peer audio format. */
   public AudioFormat getAudioFormat()
   {
      return audioFormat;
   }

   /** {@inheritDoc} */
   @Override
   public boolean equals(final Object obj)
   {
      return obj instanceof Peer && ((Peer) obj).ip.equals(ip);
   }
   
   /** {@inheritDoc} */
   @Override
   public int hashCode()
   {
      return ip.hashCode();
   }
   
   /** {@inheritDoc} */
   @Override
   public String toString()
   {
      return getName();
   }
   
   /**
    * Reads messages from this Peer.
    */
   private final class ReadThread extends Thread
   {
      /**
       * Creates a ReadThread.
       */
      public ReadThread()
      {
         setName("ClientRead_" + getIp());
      }
      
      /** {@inheritDoc} */
      @Override
      public void run()
      {
         try
         {
            String line;
            
            while ((line = read()) != null)
            {
               if (line.startsWith(MESSAGE))
               {
                  String message = line.substring(MESSAGE.length()).replace(DELIMITER, '\n');
                  
                  for (PeerListener listener : Peers.getListeners())
                  {
                     listener.peerMessage(Peer.this, message);
                  }
               }
               else if (line.startsWith(DIAL))
               {
                  if (CallManager.isCallActive())
                  {
                     send(BUSY);
                  }
                  else
                  {
                     audioFormat = AudioUtil.decodeAudioFormat(line.substring(DIAL.length()));
                     setCallState(CallState.RINGING);
                  }
               }
               else if (line.startsWith(PICKUP))
               {
                  audioFormat = AudioUtil.decodeAudioFormat(line.substring(PICKUP.length()));
                  CallManager.setActiveCall(Peer.this);
                  setCallState(CallState.CONNECTED);
               }
               else if (PING.equals(line))
               {
                  send(PONG);
               }
               else if (PONG.equals(line))
               {
                  // Ping response - ignore
               }
               else if (BUSY.equals(line))
               {
                  setCallState(Peer.CallState.BUSY);
               }
               else if (END_CALL.equals(line))
               {
                  CallManager.endCall();
                  setCallState(CallState.INACTIVE);
               }
               else
               {
                  log.log(Level.WARNING, "Invalid peer request: " + line.substring(0, Math.min(20, line.length())));
                  break;
               }
            }
         }
         catch (Exception e)
         {
            log.log(Level.INFO, "Error reading from peer", e);
         }
         
         disconnect();
      }
   }

   /**
    * Periodically sends a Ping message to check whether the Peer is still connected.
    */
   private final class PingThread extends Thread
   {
      /** 
       * Creates a PingThread. 
       */
      public PingThread()
      {
         setName("ClientKeepAlive_" + getIp());
      }
      
      /** {@inheritDoc} */
      @Override
      public void run()
      {
         for (;;)
         {
            try
            {
               Thread.sleep(PING_INTERVAL);
               send(PING);
            }
            catch (InterruptedException e)
            {
               break;
            }
         }
      }
   }
}
