package com.aleax.voip;

import java.io.IOException;
import java.lang.ref.WeakReference;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.aleax.voip.ui.AppConfig;
import com.aleax.voip.util.LogUtil;
import com.aleax.voip.util.Util;

/**
 * This class is responsible for managing peer connections.
 *  
 * @author Yiannis Paschalidis
 */
public final class Peers
{
   /** The logger instance for this class. */
   private static final Logger log = LogUtil.get(Peers.class);

   /** Poll interval, in milliseconds. */
   private static final int POLL_INTERVAL = 10000;

   /** Scan timeout, in milliseconds. */
   private static final int SINGLE_IP_SCAN_TIMEOUT = 1000;
   
   /** Used to synchronize access to Peer collections .*/
   private static final Object PEER_LOCK = new Object();
   
   /** The set of currently connected Peers' IP address, for quick access. */
   private static final Set<String> peerIps = new HashSet<String>();
   
   /** The currently connected Peers. */
   private static final List<Peer> peers = new ArrayList<Peer>();

   /** Listeners to notify for peer related events. See {@link PeerListener}. */
   private static final List<WeakReference<PeerListener>> listeners = new ArrayList<WeakReference<PeerListener>>();
   
   /** The thread responsible for searching for other peers on the network. */
   private static final SearchPeerThread searchThread = new SearchPeerThread();
   
   /** The thread responsible for processing connection requests from other Peers. */ 
   private static final AcceptPeerThread acceptThread = new AcceptPeerThread();
   
   /** Flag to indicate that the application is shutting down. */
   private static volatile boolean closed = false;
   
   /** Prevent instantiation of this class. */
   private Peers()
   {
   }

   static
   {
      String localIp = Util.getIntIp();

      if (localIp == null)
      {
         log.warning("No network found");
      }
      else
      {
         // Don't scan local ip.
         peerIps.add(localIp);
      }
   }

   /**
    * Starts the peer connection threads.
    */
   public static void start()
   {
      acceptThread.start();
      searchThread.start();
   }

   /** @return the list of currently connected Peers. */
   public static List<Peer> getCurrentPeers()
   {
      synchronized (PEER_LOCK)
      {
         return new ArrayList<Peer>(peers);
      }
   }

   /**
    * Disconnects all Peers and stops accepting new connections.
    */
   public static void stop()
   {
      closed = true;
      searchThread.interrupt();
      acceptThread.interrupt();
      
      for (Peer peer : getCurrentPeers())
      {
         if (peer.isConnected())
         {
            peer.disconnect();
         }
      }
   }

   /**
    * Adds a listener to be notified about Peer related events.
    * @param listener the listner to add.
    */
   public static void addListener(final PeerListener listener)
   {
      synchronized (listeners)
      {
         listeners.add(new WeakReference<PeerListener>(listener));
      }
   }

   /** @return the Peer listeners. */
   public static List<PeerListener> getListeners()
   {
      List<PeerListener> result = new ArrayList<PeerListener>(listeners.size());
      
      synchronized (listeners)
      {
         for (Iterator<WeakReference<PeerListener>> i = listeners.iterator() ; i.hasNext() ; )
         {
            PeerListener listener = i.next().get();
            
            if (listener == null)
            {
               i.remove();
            }
            else
            {
               result.add(listener);
            }
         }
      }
      
      return result;
   }

   /**
    * Registers a Peer as having connected.
    * @param peer the peer to add.
    */
   private static void addPeer(final Peer peer)
   {
      synchronized (PEER_LOCK)
      {
         peers.add(peer);
         peerIps.add(peer.getIp());

         for (PeerListener listener : getListeners())
         {
            try
            {
               listener.peerFound(peer);
            }
            catch (Exception e)
            {
               log.log(Level.WARNING, "Error in listener", e);
            }
         }
      }
   }

   /**
    * Registers a Peer as having disconnected.
    * @param peer the peer to remove.
    */
   static void removePeer(final Peer peer)
   {
      synchronized (PEER_LOCK)
      {
         if (peers.remove(peer))
         {
            peerIps.remove(peer.getIp());

            for (PeerListener listener : getListeners())
            {
               try
               {
                  listener.peerLost(peer);
               }
               catch (Exception e)
               {
                  log.log(Level.WARNING, "Error in listener", e);
               }
            }
         }
      }
   }

   /**
    * This thread monitors a ServerSocket for Peer connection attempts.
    */
   private static final class AcceptPeerThread extends Thread
   {
      /** The server socket to monitor. */
      private ServerSocket serverSocket;

      /** 
       * Creates an AcceptPeerThread.
       */
      public AcceptPeerThread()
      {
         setName("AcceptPeers");
      }
      
      /** {@inheritDoc} */
      @Override
      public void interrupt() 
      {
         super.interrupt();
         Util.safeClose(serverSocket);
         serverSocket = null;
      }

      /** {@inheritDoc} */
      @Override
      public void run()
      {
         try
         {
            serverSocket = new ServerSocket(AppConfig.getTcpPort());
         }
         catch (IOException e)
         {
            // TODO: Present to user?
            log.log(Level.WARNING, "Failed to start server", e);
            return;
         }

         for ( ; ;)
         {
            try
            {
               Socket socket = serverSocket.accept();

               synchronized (PEER_LOCK)
               {
                  if (peerIps.contains(socket.getInetAddress().getHostAddress()))
                  {
                     // Already connected to this peer
                     Util.safeClose(socket);
                  }
                  else
                  {
                     addPeer(new Peer(socket));
                  }
               }
            }
            catch (Exception e)
            {
               if (!closed)
               {
                  log.log(Level.WARNING, "Client connect failed", e);
               }
            }
         }
      }
   }

   /**
    * This thread searches the network for other Peers.
    */
   private static final class SearchPeerThread extends Thread
   {
      /** The maximum number of simultaneous searches. */
      private static final int MAX_THREADS = 100;

      /** Executor service used to concurrently scan IPs. */
      private ExecutorService es; 
      
      /** 
       * Creates a SearchPeerThread. 
       */
      public SearchPeerThread()
      {
         setName("SearchPeers");
      }

      /** {@inheritDoc} */
      @Override
      public void run()
      {
         try
         {
            String localIp = Util.getIntIp();
            
            if (localIp == null || "127.0.0.1".equals(localIp))
            {
               log.info("No network found");
               return;
            }
            
            while (!closed)
            {
               List<String> ipsToScan = new ArrayList<String>();
               
               if (AppConfig.isScanLocalNetwork())
               {
                  String ipPrefix = localIp.substring(0, localIp.lastIndexOf('.') + 1);
                  
                  for (int i = 1 ; i <= 255 ; i++)
                  {
                     String targetIp = ipPrefix + i;
                     
                     if (!targetIp.equals(localIp))
                     {
                        ipsToScan.add(targetIp);
                     }
                  }               
               }
               else
               {
                  ipsToScan.addAll(Arrays.asList(AppConfig.getIpsToScan()));               
               }
               
               ipsToScan.remove(localIp);

               es = Executors.newFixedThreadPool(MAX_THREADS);
               final int fullScanTimeout = ipsToScan.size() * SINGLE_IP_SCAN_TIMEOUT / MAX_THREADS; 
               
               final List<Future<Socket>> futures = new ArrayList<>();
               
               for (int i = ipsToScan.size() - 1 ; i >= 0 ; i--)
               {
                  futures.add(scanIp(es, ipsToScan.get(i)));
               }
              
               es.awaitTermination(fullScanTimeout, TimeUnit.MILLISECONDS);
               
               for (int i = futures.size() - 1 ; i >= 0 ; i--)
               {
                  Socket socket = null;
                  
                  try
                  {
                     socket = futures.get(i).get();
                     
                     if (socket != null)
                     {
                        synchronized (PEER_LOCK)
                        {
                           if (!peerIps.contains(socket.getInetAddress().getHostAddress()))
                           {
                              addPeer(new Peer(socket));
                           }
                        }
                     }
                  }
                  catch (Exception e)
                  {
                     // Don't care - most of the scan will fail
                  }
               }

               Thread.sleep(POLL_INTERVAL);
            }
         }
         catch (InterruptedException e)
         {
            if (!closed)
            {
               log.log(Level.WARNING, "Interrupted", e);
            }
         }
         finally
         {
            if (es != null)
            {
               es.shutdownNow();
               es = null;
            }
         }
      }
      
      /** {@inheritDoc} */
      @Override
      public void interrupt()
      {
         super.interrupt();
         
         if (es != null)
         {
            es.shutdownNow();
            es = null;
         }
      }

      /**
       * Creates and submits a Callable which scans a single IP address.
       * @param es the executor service which will execute the Callable.
       * @param ip the IP address to scan.
       * @return the future result of the scan.
       */
      private Future<Socket> scanIp(final ExecutorService es, final String ip)
      {
         return es.submit(new Callable<Socket>()
         {
            @Override
            public Socket call() throws Exception
            {
               if (!closed)
               {
                  Socket socket = new Socket();
                  socket.connect(new InetSocketAddress(ip, AppConfig.getTcpPort()), SINGLE_IP_SCAN_TIMEOUT);
                  return socket;
               }
               
               return null;
            }
         });
      }
   }
}
