package com.aleax.voip;

import java.io.BufferedInputStream;
import java.io.OutputStream;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Mixer;
import javax.sound.sampled.SourceDataLine;

import com.aleax.voip.ui.AppConfig;
import com.aleax.voip.util.Dual;
import com.aleax.voip.util.EncryptionUtil;
import com.aleax.voip.util.LogUtil;
import com.aleax.voip.util.Util;

/**
 * Reads audio data from a socket and sends it to the given output.
 * This thread runs for the life of the application.
 * 
 * @author Yiannis Paschalidis
 * @see SendAudioThread
 */
public class ReceiveAudioThread extends Thread
{
   /** The logger instance for this class. */
   private static final Logger log = LogUtil.get(ReceiveAudioThread.class);
   
   /** Audio buffer size - these must match. */
   private static final int AUDIOBUFFER_LENGTH = SendAudioThread.AUDIOBUFFER_LENGTH;

   /** Used to synchronize access to audio device. */
   private final Object AUDIO_LOCK = new Object();
   
   /** Used to synchronize access to the Socket. */
   private final Object SOCKET_LOCK = new Object();
   
   /** Used to synchronize access to the Peer. */
   private final Object PEER_LOCK = new Object();
   
   /** The currently connected peer, or null if there is no call in progress. */
   private Peer peer;   
   
   /** True if the audio should be encrypted. */
   private boolean encrypted;
   
   /** The socket to use for transport. */
   private DatagramSocket socket;
   
   /** The peer's audio format. */
   private AudioFormat audioFormat;
   
   /** The line to send the audio data from. */
   private SourceDataLine sourceDataLine;
   
   /** If audio format conversion is required, this stream provides the converted data. */
   private AudioInputStream conversionInputStream;
   
   /** If audio format conversion is required, this stream receives the peer's raw data. */
   private OutputStream conversionOutputStream;   
   
   /**
    * Creates a ReceiveAudioThread.
    */
   public ReceiveAudioThread()
   {
      setName("ReceiveAudio");
   }

   /**
    * Sets the socket used to transport the audio to the Peer.
    * @param socket the socket to use.
    */
   public void setSocket(final DatagramSocket socket)
   {
      synchronized (SOCKET_LOCK)
      {
         this.socket = socket;
      }
   }
   
   /**
    * Sets the Peer to send audio to. 
    * @param peer the connected peer, or null if there is no call in progress.
    */
   public void setPeer(final Peer peer)
   {
      synchronized (AUDIO_LOCK)
      {
         // Stop any existing audio i/o
         closeAudio();
         
         if (peer != null)
         {
            audioFormat = peer.getAudioFormat();
            
            Dual<Mixer.Info, AudioFormat> playbackDevice = AppConfig.getPlaybackDevice();
            
            try
            {
               // Unlikely, but try for an exact match first
               if (playbackDevice.getSecond().matches(audioFormat))
               {
                  // Play back the data directly without conversion
                  conversionOutputStream = null;
                  conversionInputStream = null;
                  sourceDataLine = AudioSystem.getSourceDataLine(playbackDevice.getSecond(), playbackDevice.getFirst());
                  sourceDataLine.open(audioFormat, AUDIOBUFFER_LENGTH);
                  sourceDataLine.start();
               }
               else if (AudioSystem.isConversionSupported(playbackDevice.getSecond(), audioFormat))
               {
                  conversionOutputStream = new PipedOutputStream();
                  PipedInputStream pis = new PipedInputStream((PipedOutputStream) conversionOutputStream);
               
                  conversionInputStream = new AudioInputStream(new BufferedInputStream(pis, AUDIOBUFFER_LENGTH), audioFormat, AudioSystem.NOT_SPECIFIED);
                  sourceDataLine = AudioSystem.getSourceDataLine(playbackDevice.getSecond(), playbackDevice.getFirst());
                  sourceDataLine.open(playbackDevice.getSecond(), AUDIOBUFFER_LENGTH);
                  sourceDataLine.start();
               }
               else
               {
                  throw new IllegalStateException("Can't convert from " + audioFormat); 
               }
            }
            catch (Exception e)
            {
               log.log(Level.WARNING, "Failed to create audio stream", e);
            } 
         }
      }
      
      synchronized (PEER_LOCK)
      {
         this.peer = peer;
         this.encrypted = peer == null ? false : peer.isEncrypted();
      }
   }
   
   /**
    * Closes the audio device.
    */
   private void closeAudio()
   {
      if (sourceDataLine != null)
      {
         sourceDataLine.close();
         Util.safeClose(sourceDataLine);
         sourceDataLine = null;
      }
      
      Util.safeClose(conversionInputStream);
      conversionInputStream = null;
      
      Util.safeClose(conversionOutputStream);
      conversionOutputStream = null;
   }

   /** {@inheritDoc} */
   @Override
   public void interrupt()
   {
      super.interrupt();
      setPeer(null);
   }

   /** {@inheritDoc} */
   @Override
   public void run()
   {
      final byte[] tempBuffer = new byte[AUDIOBUFFER_LENGTH];
      final DatagramPacket p = new DatagramPacket(tempBuffer, AUDIOBUFFER_LENGTH);

      for (;;)
      {
         try
         {
            boolean received = false;
            
            synchronized (SOCKET_LOCK)
            {
               if (socket != null)
               {
                  socket.receive(p);
                  received = true;
               }
            }
            
            if (!received)
            {
               // No socket set yet
               Thread.sleep(10);
               continue;
            }

            // Make sure we're not receiving some other junk from the network
            synchronized (PEER_LOCK)            
            {
               if (peer == null || !peer.getIp().equals(p.getAddress().getHostAddress()))
               {
                  continue;
               }
            }

            byte[] audio = tempBuffer;
            
            if (encrypted)
            {
               audio = EncryptionUtil.decryptAES(tempBuffer, "TODO"/*TODO*/);
            }
                        
            synchronized (AUDIO_LOCK)
            {
               if (conversionInputStream == null)
               {
                  sourceDataLine.write(audio, 0, AUDIOBUFFER_LENGTH);
               }
               else
               {
                  conversionOutputStream.write(audio);
                 
                  while (conversionInputStream.available() > 0)
                  {
                     int len = conversionInputStream.read(audio);
                     sourceDataLine.write(audio, 0, len);
                  }                  
               }
            }
         }
         catch (Exception e)
         {
            if (isInterrupted())
            {
               return;
            }
            else
            {
               log.log(Level.WARNING, "ReceiveAudioThread failed", e);
            }
         }
      }
   }
}
