package com.aleax.voip;

import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Mixer;
import javax.sound.sampled.TargetDataLine;

import com.aleax.voip.ui.AppConfig;
import com.aleax.voip.util.Dual;
import com.aleax.voip.util.EncryptionUtil;
import com.aleax.voip.util.LogUtil;
import com.aleax.voip.util.Util;

/**
 * Reads audio data from the input device and sends it to the specified Peer.
 * This thread runs for the life of the application.
 * 
 * @author Yiannis Paschalidis
 * @see ReceiveAudioThread
 */
public class SendAudioThread extends Thread implements Runnable
{
   /** The logger instance for this class. */
   private static final Logger log = LogUtil.get(SendAudioThread.class);
   
   /** The voice activation level. The audio level must be above this threshhold to send it to the peer. */
   public static final int VOICE_ACTIVATION_LEVEL = 50; // TODO: From config
   
   /**  The size of the audio buffer, in bytes. */
   public static final int AUDIOBUFFER_LENGTH = 1344; // TODO: From config - Must be multiple of 16 for AES
   
   /** Used to synchronize access to audio device. */
   private final Object AUDIO_LOCK = new Object();
   
   /** Used to synchronize access to the Socket. */
   private final Object SOCKET_LOCK = new Object();
   
   /** Used to synchronize access to the Peer. */
   private final Object PEER_LOCK = new Object();
   
   /** The currently connected peer, or null if there is no call in progress. */
   private Peer peer;   
   
   /** True if the audio should be encrypted. */
   private boolean encrypted;
   
   /** The socket to use for transport. */
   private DatagramSocket socket;
   
   /** The line to read the audio data from. */
   private TargetDataLine targetDataLine;
   
   /** Creates a SendAudioThread. */
   public SendAudioThread()
   {
      setName("SendAudio");
   }

   /**
    * Sets whether the audio should be encrypted.
    * @param encrypted true to encrypt the audio, false to send as is.
    */
   public void setEncrypted(final boolean encrypted)
   {
      this.encrypted = encrypted;
   }

   /**
    * Sets the socket used to transport the audio to the Peer.
    * @param socket the socket to use.
    */
   public void setSocket(final DatagramSocket socket)
   {
      synchronized (SOCKET_LOCK)
      {
         this.socket = socket;
      }
   }
   
   /**
    * Sets the Peer to send audio to. 
    * @param peer the connected peer, or null if there is no call in progress.
    */
   public void setPeer(final Peer peer)
   {
      synchronized (PEER_LOCK)
      {
         this.peer = peer;
         closeAudio();
         
         if (peer != null)
         {
            Dual<Mixer.Info, AudioFormat> recordingDevice = AppConfig.getRecordingDevice();
            
            if (targetDataLine == null)
            {
               try
               {
                  targetDataLine = AudioSystem.getTargetDataLine(recordingDevice.getSecond(), recordingDevice.getFirst());
                  targetDataLine.open(recordingDevice.getSecond());
                  targetDataLine.start();
               }
               catch (Exception e)
               {
                  // TODO: Handle exception properly
                  log.log(Level.WARNING, "Failed to open recording device", e);
               }
            }
         }
      }
   }
   
   /**
    * Closes the audio device.
    */
   private void closeAudio()
   {
      if (targetDataLine != null)
      {
         targetDataLine.stop();
         Util.safeClose(targetDataLine);
         targetDataLine = null;
      }
   }
   
   /** {@inheritDoc} */
   @Override
   public void interrupt()
   {
      super.interrupt();
      setPeer(null);
   }

   /** {@inheritDoc} */
   @Override
   public void run()
   {
      try
      {
         final byte[] audio = new byte[AUDIOBUFFER_LENGTH];
         final DatagramPacket p = new DatagramPacket(audio, AUDIOBUFFER_LENGTH);
         p.setLength(AUDIOBUFFER_LENGTH);
         
         while (!isInterrupted())
         {
            String ip = null;
            int port = -1;
            
            synchronized (PEER_LOCK)
            {
               if (peer != null)
               {
                  ip = peer.getIp();
                  port = peer.getUdpPort();
               }
            }

            if (ip == null)
            {
               Thread.sleep(100);
               continue;
            }
            
            int cnt = 0;
            
            synchronized (AUDIO_LOCK)
            {
               cnt = targetDataLine.read(audio, 0, audio.length);
            }

            int soundLevel = 0;
           
            for (int i = 0 ; i < cnt ; i++)
            {
               // TODO: Handle multi-byte!
               if ((audio[i] & (byte) 0x7f) >= VOICE_ACTIVATION_LEVEL)
               {
                  soundLevel++;
               }
            }
            
            if (true /* TODO: Causing buffer underrun */ || soundLevel > AUDIOBUFFER_LENGTH / 100) // at least 1% must be above threshold
            {
               try
               {
                  if (encrypted)
                  {
                     p.setData(EncryptionUtil.encryptAES(audio, "TODO"/*TODO*/));
                  }
                  else
                  {
                     p.setData(audio);
                  }
                  
                  p.setAddress(InetAddress.getByName(ip));                     
                  p.setPort(port);
                  
                  synchronized (SOCKET_LOCK)
                  {
                     socket.send(p);
                  }
               }
               catch (Exception e)
               {
                  // TODO: Handle exception properly - close connection?
                  log.log(Level.WARNING, "Failed to send audio data", e);
               }
            }
         }
      }
      catch (Exception e)
      {
         if (isInterrupted())
         {
            return;
         }
         else
         {
            log.log(Level.WARNING, "SendAudioThread failed", e);
         }
      }
   }
}
