package com.aleax.voip.ui;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.Mixer;

import com.aleax.voip.util.AudioUtil;
import com.aleax.voip.util.Config;
import com.aleax.voip.util.Dual;
import com.aleax.voip.util.Util;

/**
 * This utility class provides convenient getters/setters to the backing {@link Config}. 
 * @author Yiannis Paschalidis
 */
public final class AppConfig
{
   /** The configuration file name. */
   private static final String CONFIG_FILE_NAME = "voip.conf";
   
   /** Backing configuration instance. */
   private static final Config CONFIG = Config.getInstance(CONFIG_FILE_NAME, false);

   private static final String ENCRYPTED = "encrypt";
   private static final String USE_PASSWORD = "use_password";
   private static final String TCP_PORT = "tcp_port";
   private static final String UDP_PORT = "udp_port";
   private static final String PASSWORD = "password";
   private static final String NICKNAME = "nickname";
   private static final String RECORDING_DEVICE = "recording";
   private static final String PLAYBACK_DEVICE = "playback";
   private static final String SCAN_LOCAL_NET = "scanLocalNet";
   private static final String SCAN_IPS = "scanIps";
   
   /** Prevent instantiation of this utility class. */
   private AppConfig()
   {
   }
   
   /** Saves the current configuration to disk. */
   public static void save()
   {
      CONFIG.write(CONFIG_FILE_NAME);
   }

   /** @return the connection password. */
   public static String getPassword()
   {
      return CONFIG.get(PASSWORD);
   }
   
   /**
    * Sets the connection password.
    * @param password the password to set.
    */
   public static void setPassword(final String password)
   {
      CONFIG.set(PASSWORD, password);
   }  
   
   /** @return the TCP port which is used for p2p text comms. */
   public static int getTcpPort()
   {
      return CONFIG.getInt(TCP_PORT, 9998);
   }
   
   /** 
    * Sets the TCP port which is used for p2p text comms.
    * @param port the port to set.
    */
   public static void setTcpPort(final int port)
   {
      CONFIG.set(TCP_PORT, port);
   }
   
   /** @return the UDP port used for voice chat. */
   public static int getUdpPort()
   {
      return CONFIG.getInt(UDP_PORT, 9999);
   }
   
   /**
    * Sets the UDP port used for voice chat.
    * @param port the port to set.
    */
   public static void setUdpPort(final int port)
   {
      CONFIG.set(UDP_PORT, port);
   }
   
   /** @return the user's nickname. */
   public static String getNickname()
   {
      return CONFIG.get(NICKNAME, System.getProperty("user.name", "unknown"));
   }
   
   /**
    * Sets the user's nickname.
    * @param nickname the name to set.
    */
   public static void setNickname(final String nickname)
   {
      CONFIG.set(NICKNAME, nickname);
   }

   /** @return true if communications should be encrypted. */
   public static boolean isEncrypted()
   {
      return CONFIG.getBoolean(ENCRYPTED, false);
   }
   
   /**
    * Sets whether communications should be encrypted.
    * @param encrypted true for encrypted communications, false for unencrypted.
    */
   public static void setEncrypted(final boolean encrypted)
   {
      CONFIG.set(ENCRYPTED, encrypted);
   }
   
   /** @return the device used for audio input. */ 
   public static Dual<Mixer.Info, AudioFormat> getRecordingDevice()
   {
      return AudioUtil.decodeAudioDevice(CONFIG.get(RECORDING_DEVICE));
   }
   
   /**
    * Sets the device used for audio input.
    * @param device the device settings.
    */
   public static void setRecordingDevice(final Dual<Mixer.Info, AudioFormat> device)
   {
      CONFIG.set(RECORDING_DEVICE, AudioUtil.encodeAudioDevice(device, true));
   }
   
   /** @return the device used for audio output. */ 
   public static Dual<Mixer.Info, AudioFormat> getPlaybackDevice()
   {
      return AudioUtil.decodeAudioDevice(CONFIG.get(PLAYBACK_DEVICE));
   }
   
   /**
    * Sets the device used for audio output.
    * @param device the device settings.
    */
   public static void setPlaybackDevice(final Dual<Mixer.Info, AudioFormat> device)
   {
      CONFIG.set(PLAYBACK_DEVICE, AudioUtil.encodeAudioDevice(device, false));
   }

   /** @return true if the entire local network should be scanned. */
   public static boolean isScanLocalNetwork()
   {
      return CONFIG.getBoolean(SCAN_LOCAL_NET, true);
   }
   
   /**
    * Sets whether the entire local network should be scanned.
    * @param enabled true to enable the network scan, false to scan individual IP addresses.
    */
   public static void setScanLocalNetwork(final boolean enabled)
   {
      CONFIG.set(SCAN_LOCAL_NET, enabled);
   }
   
   /**
    * @return the IP addresses to scan for Peers.
    */
   public static String[] getIpsToScan()
   {
      String setting = CONFIG.get(SCAN_IPS, "");
      return Util.empty(setting) ? new String[0] : setting.split(",");
   }
   
   /**
    * Sets the IP addresses to scan for Peers.
    * @param ips the IP addresses to set.
    */
   public static void setIpsToScan(final String[] ips)
   {
      if (ips != null && ips.length > 0)
      {
         StringBuilder buf = new StringBuilder(ips[0]);
         
         for (int i = 1 ; i < ips.length ; i++)
         {
            buf.append(',').append(ips[i]);
         }
         
         CONFIG.set(SCAN_IPS, buf.toString());
      }
      else
      {
         CONFIG.set(SCAN_IPS, null);
      }
   }
}
