package com.aleax.voip.ui;

import java.awt.FlowLayout;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import com.aleax.voip.Peer;
import com.aleax.voip.Peer.CallState;
import com.aleax.voip.PeerListener;
import com.aleax.voip.Peers;
import com.aleax.voip.util.Util;

/**
 * Presents buttons for initiating/accepting/ending VOIP calls.
 * 
 * @author Yiannis Paschalidis
 */
public final class CallButtons extends JPanel
{
   /** The size of each button in pixels. */
   private static final int BUTTON_SIZE = 64;
   
   /** The peer associated with this button. */
   private Peer peer;
   
   private final JButton acceptButton = createButton("/images/call_accept.png");
   private final JButton rejectButton = createButton("/images/call_reject.png");
   private final JButton callButton = createButton("/images/call_peer.png");
   private final JButton hangupButton = createButton("/images/call_end.png"); 
   
   /** Creates a CallButton. */
   public CallButtons()
   {
      setLayout(new FlowLayout(FlowLayout.LEFT, BUTTON_SIZE / 10, 0));
      add(acceptButton);
      add(rejectButton);
      add(callButton);
      add(hangupButton);
      
      acceptButton.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(final ActionEvent e)
         {
            peer.pickup();
         }
      });
      
      rejectButton.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(final ActionEvent e)
         {
            peer.hangup();
         }
      });      
      
      hangupButton.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(final ActionEvent e)
         {
            peer.hangup();
         }
      });      
      
      callButton.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(final ActionEvent e)
         {
            peer.dial();
         }
      });
      
      Peers.addListener(new PeerListener()
      {
         @Override
         public void peerMessage(final Peer peer, final String message)
         {
            // NOP
         }
         
         @Override
         public void peerLost(final Peer peer)
         {
            if (Util.equals(peer, CallButtons.this.peer))
            {
               updateState();
            }
         }
         
         @Override
         public void peerFound(final Peer peer)
         {
            if (Util.equals(peer, CallButtons.this.peer))
            {
               updateState();
            }
         }
         
         @Override
         public void peerCallStatusChanged(final Peer peer, final CallState fromState)
         {
            if (Util.equals(peer, CallButtons.this.peer))
            {
               updateState();
            }
         }
      });
      
      updateState();
   }
   
   /**
    * Sets the peer associated with this button.
    * @param peer the peer to set.
    */
   public void setPeer(final Peer peer)
   {
      this.peer = peer;
      updateState();
   }
   
   /**
    * Updates the panel's state.
    */
   public void updateState()
   {
      if (peer == null || !peer.isConnected())
      {
         callButton.setEnabled(false);
         
         acceptButton.setVisible(false);
         rejectButton.setVisible(false);
         callButton.setVisible(true);
         hangupButton.setVisible(false);
      }
      else
      {
         callButton.setEnabled(true);
         
         switch (peer.getCallState())
         {
            case CONNECTED:
            {
               acceptButton.setVisible(false);
               rejectButton.setVisible(false);
               callButton.setVisible(false);
               hangupButton.setVisible(true);
               break;
            }
            case DIALLING:
            {
               acceptButton.setVisible(false);
               rejectButton.setVisible(false);
               callButton.setVisible(false);
               hangupButton.setVisible(true);
               break;
            }  
            case BUSY:
            {            
               acceptButton.setVisible(false);
               rejectButton.setVisible(false);
               callButton.setVisible(true);
               hangupButton.setVisible(false);
               break;
            }
            case INACTIVE:
            {            
               acceptButton.setVisible(false);
               rejectButton.setVisible(false);
               callButton.setVisible(true);
               hangupButton.setVisible(false);
               break;
            }
            case RINGING:
            {
               acceptButton.setVisible(true);
               rejectButton.setVisible(true);
               callButton.setVisible(false);
               hangupButton.setVisible(false);
               break;
            }
         }
      }
      
      SwingUtilities.invokeLater(new Runnable()
      {
         @Override
         public void run()
         {
            repaint();
         }
      });
   }
   
   /**
    * Creates a button with the given image.
    * @param imageUrl the image resource url.
    * @return the button.
    */
   private static JButton createButton(final String imageUrl)
   {
      Image image = Util.getImage(imageUrl).getScaledInstance(BUTTON_SIZE, BUTTON_SIZE, Image.SCALE_SMOOTH);
      return new JButton(new ImageIcon(image));
   }
}
