package com.aleax.voip.ui;

import java.io.BufferedInputStream;
import java.io.InputStream;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Clip;
import javax.sound.sampled.Mixer;

import com.aleax.voip.util.Dual;
import com.aleax.voip.util.LogUtil;

/**
 * Manages application sounds.
 * @author Yiannis Paschalidis.
 */
public final class CallSounds
{
   /** The logger instance for this class. */
   private static final Logger log = LogUtil.get(CallSounds.class);
   
   /** Enumeration of available sounds. */
   public enum Sound
   {
      RINGING("/sounds/ringing.wav"),
      DIALLING("/sounds/dialling.wav"),
      CALL_FAILED("/sounds/call_failed.wav"),
      CALL_ENDED("/sounds/call_disconnect.wav") // TODO
      ;
      
      /** The resource path for the sound file. */
      private final String path;
      
      /**
       * Creates a sound.
       * @param path the resource path for the sound file.
       */
      private Sound(final String path)
      {
         this.path = path;
      }
      
      /** @return the resource path for the sound file. */
      public String getPath()
      {
         return path;
      }
   }

   /** Clip holding the current sounds which is playing. */
   private static Clip clip;

   /** Used to synchronize access to {@link #Clip}. */
   private static final Object CLIP_LOCK = new Object();
   
   /** Prevent instantiation of this utility class. */
   private CallSounds()
   {
   }
   
   /**
    * Plays the given sounds.
    * @param sound the sound to play.
    */
   public static void play(final Sound sound)
   {
      synchronized (CLIP_LOCK)
      {
         clear();
         
         clip = getClip(sound);
         
         if (clip != null)
         {
            clip.start();
         }
      }
   }
   
   /**
    * Loops the given sound until another is played, or {@link #clear()} is called.
    * @param sound the sound to loop
    */
   public static void loop(final Sound sound)
   {
      synchronized (CLIP_LOCK)
      {
         clear();
         
         clip = getClip(sound);
         
         if (clip != null)
         {
            clip.loop(Clip.LOOP_CONTINUOUSLY);
         }      
      }
   }
   
   /**
    * Obtains a clip for playback.
    * @param sound the sound to play.
    * @return the sound Clip.
    */
   private static Clip getClip(final Sound sound)
   {
      // TODO: Cache clips
      // TODO: Mixer selection
      try
      {
         Dual<Mixer.Info, AudioFormat> playbackDevice = AppConfig.getPlaybackDevice();
         
         InputStream is = new BufferedInputStream(CallSounds.class.getResourceAsStream(sound.getPath()));
         AudioInputStream stream = AudioSystem.getAudioInputStream(is);
         clip = playbackDevice == null ? AudioSystem.getClip() : AudioSystem.getClip(playbackDevice.getFirst());
         clip.open(stream);
         clip.setFramePosition(0);
         return clip;
      }
      catch (Exception e)
      {
         log.log(Level.WARNING, "Failed to open clip: " + sound.name(), e);
         return null;
      }
   }

   /**
    * Stops the currently playing sound, if any.
    */
   public static void clear()
   {
      synchronized (CLIP_LOCK)
      {
         if (clip != null)
         {
            clip.stop();
            clip.close();
            clip = null;
         }
      }
   }
}
