package com.aleax.voip.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.JTextPane;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;

import com.aleax.voip.Peer;
import com.aleax.voip.util.AutoScrollPane;
import com.aleax.voip.util.XPanel;
import com.aleax.voip.util.XSplitPane;

/**
 * Allows the user to interact with a Peer (chat/voice).
 * @author Yiannis Paschalidis
 */
public class PeerPanel extends XPanel
{
   /** Style for dates/times. */
   private static final SimpleAttributeSet DATE_STYLE;
   
   /** Style for displaying the user's name. */
   private static final SimpleAttributeSet USER_NAME_STYLE;
   
   /** Style for displaying the peer's name. */
   private static final SimpleAttributeSet PEER_NAME_STYLE;
   
   /** Style for Info messages from the application. */
   private static final SimpleAttributeSet INFO_TEXT_STYLE;
   
   /** Style for user/peer messages. */
   private static final SimpleAttributeSet MESSAGE_TEXT_STYLE;
   
   static
   {
      DATE_STYLE = new SimpleAttributeSet();
      StyleConstants.setForeground(DATE_STYLE, new Color(120, 120, 120));                
      
      USER_NAME_STYLE = new SimpleAttributeSet();
      StyleConstants.setForeground(USER_NAME_STYLE, new Color(0, 0, 80));
      StyleConstants.setBold(USER_NAME_STYLE, true);
      
      PEER_NAME_STYLE = new SimpleAttributeSet();
      StyleConstants.setForeground(PEER_NAME_STYLE, new Color(250, 120, 0));
      StyleConstants.setBold(PEER_NAME_STYLE, true);
      
      INFO_TEXT_STYLE = new SimpleAttributeSet();
      StyleConstants.setForeground(INFO_TEXT_STYLE, new Color(120, 120, 120));                
      StyleConstants.setBold(INFO_TEXT_STYLE, true);
      
      MESSAGE_TEXT_STYLE = new SimpleAttributeSet();
   }
   
   /** The Peer to interact with. */
   private Peer peer;
   
   /** Displays messages and events. */
   private final JTextPane display = new JTextPane();

   /** Used to synchronize access to the display and date formats. */
   private final Object DISPLAY_LOCK = new Object();

   /** Used for formatting dates. */
   private final SimpleDateFormat dateFormat = new SimpleDateFormat("dd MMM yyyy");
   
   /** Used for formatting times. */
   private final SimpleDateFormat timeFormat = new SimpleDateFormat("HH:mm:ss");
   
   /** Allows the user to enter a text message to send to the peer. */
   private final JTextArea input = new JTextArea();
   
   /** Allows the user to initiate, accept and terminate VOIP calls. */
   private final CallButtons callButtons = new CallButtons();
   
   /** Displays status text. */
   private final JLabel statusLabel = new JLabel(" ");
   
   /** Indicates whether ther peer is currently connected. */
   private boolean connected = false;
   
   /** When the last message was appended. */
   private long lastMessageTime = 0;

   /**
    * Creates a PeerPanel.
    * @param peer the Peer to interact with.
    */
   public PeerPanel(final Peer peer)
   {
      setPeer(peer);
      display.setEditable(false);
      
      XSplitPane splitter = new XSplitPane(JSplitPane.VERTICAL_SPLIT);
      
      JPanel topPanel = new JPanel();
      topPanel.setLayout(new BorderLayout());
      topPanel.add(new JLabel("Chat history"), BorderLayout.NORTH);
      topPanel.add(new AutoScrollPane(display, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER), BorderLayout.CENTER);
      splitter.setTopComponent(topPanel);
      
      JPanel bottomPanel = new JPanel();
      bottomPanel.setLayout(new BorderLayout());
      bottomPanel.add(new JLabel("Message to send"), BorderLayout.NORTH);
      bottomPanel.add(new JScrollPane(input, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER), BorderLayout.CENTER);
      splitter.setBottomComponent(bottomPanel);
      
      addFillXY(splitter, 0, 0, 2, 1);      
      addFillX(statusLabel, 0, 1);
      add(callButtons, 1, 1);

      splitter.setDividerLocation(0.5);
      
      input.addKeyListener(new KeyAdapter()
      {
         @Override
         public void keyPressed(final KeyEvent e)
         {
            if (e.getKeyCode() == KeyEvent.VK_ENTER)
            {
               e.consume();

               if ((e.getModifiers() & KeyEvent.SHIFT_MASK) != 0)
               {
                  input.append("\n");
               }
               else
               {
                  String message = input.getText().trim();
                  
                  if (message != null && message.length() > 0)
                  {
                     PeerPanel.this.peer.sendMessage(message);
                     appendUserMessage(message);
                     input.setText("");
                  }
               }
            }
         }
      });
   }
   
   /**
    * Sets the status text.
    * @param status the text to set.
    */
   public void setStatus(final String status)
   {
      statusLabel.setText(" " + status);
   }
   
   /**
    * Appends a message from the peer to the display area.
    * @param message the message to append.
    */
   public void appendMessage(final String message)
   {
      synchronized (DISPLAY_LOCK)
      {
         appendTimestamp();
         append("<", MESSAGE_TEXT_STYLE);
         append(peer.getName(), PEER_NAME_STYLE);
         append("> " + message + "\n", MESSAGE_TEXT_STYLE);
      }
   }

   /**
    * Appends a message from the user to the display area.
    * @param message the message to append.
    */
   private void appendUserMessage(final String message)
   {
      synchronized (DISPLAY_LOCK)
      {
         appendTimestamp();
         append("<", MESSAGE_TEXT_STYLE);
         append(AppConfig.getNickname(), USER_NAME_STYLE);
         append("> " + message + "\n", MESSAGE_TEXT_STYLE);
      }
   }
   
   /**
    * Appends an application-generated information message to the display.
    * @param message the message to append.
    */
   public void appendInfoMessage(final String message)
   {
      synchronized (DISPLAY_LOCK)
      {
         appendTimestamp();
         append(message + "\n", INFO_TEXT_STYLE);
      }
   }
   
   /**
    * Appends the current time to the display.
    */
   private void appendTimestamp()
   {
      Calendar cal = GregorianCalendar.getInstance();
      Date now = cal.getTime();
      
      int currentDay = cal.get(Calendar.YEAR) * 1000 + cal.get(Calendar.DAY_OF_YEAR);
      
      cal.setTimeInMillis(lastMessageTime);
      int lastDay = cal.get(Calendar.YEAR) * 1000 + cal.get(Calendar.DAY_OF_YEAR);
      
      if (currentDay != lastDay)
      {
         append("-------- " + dateFormat.format(now) + " --------\n", DATE_STYLE);
      }
      
      append(timeFormat.format(now) + ": ", DATE_STYLE);
      lastMessageTime = now.getTime();
   }
   
   /**
    * Appends the given text to the display.
    * @param text the text to append.
    * @param style the text style, or null to use the default styling.
    */
   private void append(final String text, final SimpleAttributeSet style)
   {
      Document doc = display.getDocument();
      
      try
      {
         doc.insertString(doc.getLength(), text, style);
      }
      catch (BadLocationException e)
      {
         // Not bad location
      }
   }
   
   /**
    * Sets the peer. This is used for recovery from lost connections.
    * @param peer the peer to set.
    */
   public void setPeer(final Peer peer)
   {
      this.peer = peer;
      callButtons.setPeer(peer);
      updateConnectionStatus();
   }
   
   /** @return the Peer. */
   public Peer getPeeer()
   {
      return peer;
   }

   /**
    * Updates the UI state according to the Peer connection status.
    */
   public void updateConnectionStatus()
   {
      boolean wasConnected = connected;
      connected = peer.isConnected();
      
      input.setEnabled(connected);
      
      if (wasConnected && !connected)
      {
         appendInfoMessage("Disconnected");
      }
      else if (!wasConnected && connected)
      {
         appendInfoMessage("Connected");
      }
   }
}
