package com.aleax.voip.ui;

import java.awt.Component;
import java.awt.Desktop;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.net.URI;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.UIManager;
import javax.swing.plaf.nimbus.NimbusLookAndFeel;

import com.aleax.voip.Peer;
import com.aleax.voip.PeerListener;
import com.aleax.voip.Peers;
import com.aleax.voip.ui.settings.SettingsDialog;
import com.aleax.voip.ui.settings.SettingsWizardDialog;
import com.aleax.voip.util.LogUtil;
import com.aleax.voip.util.Util;
import com.aleax.voip.util.XPanel;
import com.aleax.voip.util.XTabbedPane;
import com.aleax.voip.util.XTabbedPane.TabCloseListener;

/**
 * The main entry point to the VOIP application.
 * @author Yiannis Paschalidis.
 */
public class Voip2p extends JFrame implements PeerListener
{
   /** App name. */
   public static final String APP_NAME = "Voip2p";
   
   /** App version. */
   public static final String APP_VERSION = "0.1.0";
   
   /** The logger instance for this class. */
   private static final Logger log = LogUtil.get(Voip2p.class);
   
   /** Indicates whether the app window has focus. */
   private boolean windowHasFocus;
   
   /** Chat windows keyed by peer. */
   private final Map<Peer, PeerPanel> chatWindows = new HashMap<Peer, PeerPanel>();
   
   /** The tabbed pane which holds the chat windows. */
   private final XTabbedPane tabbedPane = new XTabbedPane();
   
   /** The list used to view/select peers. */
   private final JList<Peer> contactList = new JList<Peer>();
   
   /** Displays text in the application status bar. */
   private final JLabel statusBar = new JLabel("Version " + APP_VERSION + ", ready");

   /** Used to change application settings. */
   private final SettingsDialog settingsDialog = new SettingsDialog(this);
   
   /**
    * Creates a Voip2p.
    */
   public Voip2p()
   {
      setIconImages(Arrays.asList(new Image[]
      {
         Util.getImage("/images/voip2p_logo_32.png"),
         Util.getImage("/images/voip2p_logo_64.png"),
         Util.getImage("/images/voip2p_logo_128.png"),
      }));

      setTitle(APP_NAME);
      setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
      
      addWindowListener(new WindowAdapter()
      {
         @Override
         public void windowClosing(final WindowEvent e)
         {
            exit();
         }
         
         @Override
         public void windowActivated(final WindowEvent e)
         {
            windowHasFocus = true;
         }

         @Override
         public void windowDeactivated(final WindowEvent e)
         {
            windowHasFocus = false;
         }         
      });
      
      XPanel contentPane = new XPanel();
      setContentPane(contentPane);
      
      JSplitPane splitter = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
      splitter.setLeftComponent(new JScrollPane(contactList));
      splitter.setRightComponent(tabbedPane);

      contentPane.addFillXY(splitter, 0, 0);
      contentPane.addFillX(statusBar, 0, 1);
      
      contactList.addMouseListener(new MouseAdapter()
      {
         @Override
         public void mouseClicked(final MouseEvent me)
         {
            if (me.getClickCount() == 2)
            {
               int index = contactList.locationToIndex(me.getPoint());
               
               if (index >= 0)
               {
                  Peer peer = contactList.getModel().getElementAt(index);
                  getPeerPanel(peer, true);
               }
            }
         }
      });
      
      tabbedPane.addListener(new TabCloseListener()
      {
         @Override
         public void tabClosed(final Component tab)
         {
            if (tab instanceof PeerPanel)
            {
               PeerPanel panel = (PeerPanel) tab;
               chatWindows.remove(panel.getPeeer());
            }
         }
      });

      Peers.addListener(this);
      Peers.start();
      
      // TODO: Load/save position in config
      setSize(800, 600);
      Util.centerWindow(this, null);
      
      buildMenu();
      
      setVisible(true);
   }

   /**
    * Builds the application menu.
    */
   private void buildMenu()
   {
      JMenuBar menuBar = new JMenuBar();
      setJMenuBar(menuBar);
      
      JMenu appMenu = new JMenu(APP_NAME);
      menuBar.add(appMenu);

      JMenuItem settingsItem = new JMenuItem("Settings");
      settingsItem.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(final ActionEvent e)
         {
            settingsDialog.display();
         }
      });
      
      JMenuItem exitItem = new JMenuItem("Exit");
      exitItem.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(final ActionEvent e)
         {
            exit();
         }
      });

      appMenu.add(settingsItem);
      appMenu.addSeparator();
      appMenu.add(exitItem);
      
      JMenu helpMenu = new JMenu("Help");
      menuBar.add(helpMenu);

      JMenuItem helpItem = new JMenuItem("Help page");
      helpItem.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(final ActionEvent e)
         {
            if (Desktop.isDesktopSupported())
            {
               try
               {
                  Desktop.getDesktop().browse(new URI("http://aleax.com/"));
               }
               catch (Exception ex)
               {
                  log.log(Level.WARNING, "Failed to open help", ex);
               }
            }
         }
      });
      
      helpMenu.add(helpItem);
      helpMenu.addSeparator();      
      
      JMenuItem aboutItem = new JMenuItem("About");
      aboutItem.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(final ActionEvent e)
         {
            JOptionPane.showMessageDialog(Voip2p.this, new AboutPanel(), "About " + APP_NAME, JOptionPane.PLAIN_MESSAGE);
         }
      });
      
      helpMenu.add(aboutItem);
   }
   
   /**
    * Cleanly shuts down the application.
    */
   private void exit()
   {
      setVisible(false);
      AppConfig.save();
      Peers.stop();
      System.exit(0);
   }
   
   /** {@inheritDoc} */
   @Override
   public void peerCallStatusChanged(final Peer peer, final Peer.CallState fromState)
   {
      switch (peer.getCallState())
      {
         case BUSY:
         {
            CallSounds.play(CallSounds.Sound.CALL_FAILED);
            break;
         }
         case CONNECTED:
         {
            CallSounds.play(CallSounds.Sound.CALL_FAILED);
            break;
         }  
         case DIALLING:
         {
            CallSounds.loop(CallSounds.Sound.DIALLING);
            break;
         }
         case INACTIVE:
         {
            CallSounds.clear();
            
            if (Peer.CallState.CONNECTED.equals(fromState))
            {
               CallSounds.play(CallSounds.Sound.CALL_ENDED);
            }
            else if (Peer.CallState.DIALLING.equals(fromState) || Peer.CallState.RINGING.equals(fromState))
            {
               CallSounds.play(CallSounds.Sound.CALL_FAILED);
            }
                  
            break;
         }  
         case RINGING:
         {
            // Create panel for peer if one doesn't exist
            getPeerPanel(peer, true);
            CallSounds.loop(CallSounds.Sound.RINGING);
            break;
         }
      }
   }
   
   /** {@inheritDoc} */
   @Override
   public void peerFound(final Peer peer)
   {
      updatePeerList();
      
      PeerPanel panel = getPeerPanel(peer, false);
      
      if (panel != null)
      {
         panel.setPeer(peer);
      }      
   }
   
   /** {@inheritDoc} */
   @Override
   public void peerLost(final Peer peer)
   {
      updatePeerList();
      
      PeerPanel panel = getPeerPanel(peer, false);
      
      if (panel != null)
      {
         panel.updateConnectionStatus();
      }
   }
   
   /** {@inheritDoc} */
   @Override
   public void peerMessage(final Peer peer, final String message)
   {
      PeerPanel panel = getPeerPanel(peer, true);
      panel.appendMessage(message);
      
      if (!windowHasFocus)
      {
         Toolkit.getDefaultToolkit().beep();
      }
   }

   /**
    * Obtains a display panel for the given Peer.
    * 
    * @param peer the Peer to obtain the panel for.
    * @param create true to create the panel if one does not already exist.
    * @return the panel for the Peer, or null if one does not exist and <code>create</code> is false.
    */
   private PeerPanel getPeerPanel(final Peer peer, final boolean create)
   {
      PeerPanel panel = chatWindows.get(peer);
      
      if (panel == null && create)
      {
         panel = new PeerPanel(peer);
         chatWindows.put(peer, panel);
         tabbedPane.add(peer.getName() + " (" + peer.getIp() + ")", panel);
      }
      
      return panel;
   }
   
   /**
    * Updates the peer list whenever peers are added/removed.
    */
   private void updatePeerList()
   {
      Peer selected = contactList.getSelectedValue();
      List<Peer> currentPeers = Peers.getCurrentPeers();      
      contactList.setListData(currentPeers.toArray(new Peer[currentPeers.size()]));
      
      if (currentPeers.contains(selected))
      {
         contactList.setSelectedValue(selected, false);
      }
   }
   
   /**
    * Application entry point.
    * @param args ignored.
    */
   public static void main(final String[] args)
   {
      log.info("Starting up");
      
      try
      {
         UIManager.setLookAndFeel(NimbusLookAndFeel.class.getName());
      }
      catch (Exception ignored)
      {
         // Don't care - will default to cross platform plaf
         log.info("Failed to set look and feel");
      }

      if (AppConfig.getRecordingDevice() == null || AppConfig.getPlaybackDevice() == null)
      {
         new SettingsWizardDialog(null).display();
      }
      else
      {
         new Voip2p();
      }
   }
}
