package com.aleax.voip.ui.settings;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ContainerEvent;
import java.awt.event.ContainerListener;
import java.util.LinkedList;
import java.util.List;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JRadioButton;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.JTextComponent;

import com.aleax.voip.util.ValidationError;
import com.aleax.voip.util.XPanel;

/**
 * Base class for the application settings panels.
 * 
 * @author Yiannis Paschalidis
 */
public abstract class AbstractSettingsPanel extends XPanel
{
   /** The controller which receives notifications whenever a change is made on this panel. */
   protected ChangeListener controller;
   
   /** Indicates that there are unsaved changes. */
   private boolean dirty = false;
   
   /** Indicate swhether the panel has been set-up. */
   protected boolean setup = true;
   
   /** Used to listen to document events. */
   private final DocumentListener docListener = new DocumentListener()
   {
      /** {@inheritDoc} */
      @Override
      public void changedUpdate(final DocumentEvent e)
      {
         processInput();
      }

      /** {@inheritDoc} */
      @Override
      public void insertUpdate(final DocumentEvent e)
      {
         processInput();
      }

      /** {@inheritDoc} */
      @Override
      public void removeUpdate(final DocumentEvent e)
      {
         processInput();
      }
   };
   
   /** Used to listen to action events. */
   private final ActionListener actionListener = new ActionListener()
   {
      @Override
      public void actionPerformed(final ActionEvent e)
      {
         processInput();
      }
   };

   /**
    * Creates an AbstractSettingsPanel.
    */
   public AbstractSettingsPanel()
   {
      addContainerListener(new ContainerListener()
      {
         @Override
         public void componentAdded(final ContainerEvent e)
         {
            addListeners(e.getChild());
         }

         @Override
         public void componentRemoved(final ContainerEvent e)
         {
            removeListeners(e.getChild());
         }
      });
   }
   
   /** Saves the entered info to the application configuration. */
   public abstract void save();
   
   /** Resets the panel contents to those from application configuration. */
   public abstract void reset();
   
   /** Validates the panel content, adding errors to the given list. */
   public abstract void doValidate(List<ValidationError> errors);
  
   /** 
    * Validates the panel content
    * @return true if the contents are valid, false otherwise
    */
   public boolean validateInput()
   {
      clearErrors(this);
      
      if (!setup)
      {
         return false;
      }
      
      List<ValidationError> errors = new LinkedList<ValidationError>();
      doValidate(errors);
      
      for (ValidationError error : errors)
      {
         if (error.getField() instanceof Component)
         {
            Component field = (Component)error.getField();
            field.setBackground(new ErrorColor(field.getBackground()));
            
            if (field instanceof JComponent)
            {
               ((JComponent)field).setToolTipText(error.getText());            
            }
         }         
      }
      
      return errors.isEmpty();
   }

   /**
    * Recursively clears validation errors.
    * @param node the branch of the component tree to clear.
    */
   private void clearErrors(final Container node)
   {
      for (Component c : node.getComponents())
      {
         Color color = c.getBackground();

         if (color instanceof ErrorColor)
         {
            c.setBackground(((ErrorColor) color).originalColor);

            if (c instanceof JComponent)
            {
               ((JComponent) c).setToolTipText("");
            }
         }

         if (c instanceof Container)
         {
            clearErrors((Container) c);
         }
      }
   }
   
   /**
    * Sets the controller which receives notifications whenever a change is made on this panel.
    * @param controller the controller to set.
    */
   public void setController(final ChangeListener controller)
   {
      this.controller = controller;
   }

   /**
    * @return true if there are unsaved changes on this panel.
    */
   public boolean isDirty()
   {
      return dirty;
   }
   
   /**
    * Sets whether there are unsaved changes on this panel.
    * @param dirty true if there are unsaved changes, false otherwise.
    */
   public void setDirty(final boolean dirty)
   {
      this.dirty = dirty;
      
      if (controller != null)
      {
         controller.stateChanged(new ChangeEvent(this));         
      }
   }
   
   /**
    * Registers this panel to receive notifications whenever an edit is made.
    * @param c the component which was added.
    */
   private void addListeners(final Component c)
   {
      if (c instanceof JTextComponent)
      {
         ((JTextComponent) c).getDocument().addDocumentListener(docListener);
      }
      else if (c instanceof JComboBox)
      {
         ((JComboBox<?>) c).addActionListener(actionListener);
      }
      else if (c instanceof JCheckBox)
      {
         ((JCheckBox) c).addActionListener(actionListener);
      }
      else if (c instanceof JRadioButton)
      {
         ((JRadioButton) c).addActionListener(actionListener);
      }

      if (c instanceof Container && !(c instanceof AbstractSettingsPanel))
      {
         Container parent = (Container) c;

         for (int i = parent.getComponentCount() - 1 ; i >= 0 ; i--)
         {
            addListeners(parent.getComponent(i));
         }
      }
   }
   
   /**
    * De-registers this panel from receving notifications whenever an edit is made.
    * @param c the component which was removed.
    */
   private void removeListeners(final Component c)
   {
      if (c instanceof JTextComponent)
      {
         ((JTextComponent) c).getDocument().removeDocumentListener(docListener);
      }
      else if (c instanceof JComboBox)
      {
         ((JComboBox<?>) c).removeActionListener(actionListener);
      }
      else if (c instanceof JCheckBox)
      {
         ((JCheckBox) c).removeActionListener(actionListener);
      }
      else if (c instanceof JRadioButton)
      {
         ((JRadioButton) c).removeActionListener(actionListener);
      }

      if (c instanceof Container && !(c instanceof AbstractSettingsPanel))
      {
         Container parent = (Container) c;

         for (int i = parent.getComponentCount() - 1 ; i >= 0 ; i--)
         {
            removeListeners(parent.getComponent(i));
         }
      }
   }

   private void processInput()
   {
      setDirty(true);
      validateInput();
   }

   /**
    * Used to mark fields as being invalid.
    */
   private static final class ErrorColor extends Color
   {
      /** The field's original colour. */
      final Color originalColor;
      
      /**
       * Creates an ErrorColor.
       * @param originalColor the field's original colour.
       */
      public ErrorColor(final Color originalColor)
      {
         super(0xffffc0c0);
         this.originalColor = originalColor;
      }
   }
}