package com.aleax.voip.ui.settings;

import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.InetAddress;
import java.util.List;
import java.util.StringTokenizer;
import java.util.regex.Pattern;

import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.border.EmptyBorder;

import com.aleax.voip.ui.AppConfig;
import com.aleax.voip.ui.Voip2p;
import com.aleax.voip.util.IntegerTextField;
import com.aleax.voip.util.Util;
import com.aleax.voip.util.ValidationError;

/**
 * Allows the user to change network-related settings.
 * @author Yiannis Paschalidis
 */
public class NetworkSettingsPanel extends AbstractSettingsPanel
{
   /** Allows the user to set the UDP port used for voice comms. */
   private final IntegerTextField udpPort = new IntegerTextField(0, 99999);
   
   /** Allows the user to set the TCP port used for application comms. */
   private final IntegerTextField tcpPort = new IntegerTextField(0, 99999);
   
   private final JRadioButton scanLocalNet = new JRadioButton("Search local network");
   private final JRadioButton scanSpecificIps = new JRadioButton("Search specific IP addresses    ");
   private final JTextArea specificIps = new JTextArea();
   
   /**
    * Creates a NetworkSettingsPanel.
    */
   public NetworkSettingsPanel()
   {
      int y = 0;
      
      ButtonGroup group = new ButtonGroup();
      group.add(scanLocalNet);
      group.add(scanSpecificIps);
      
      ActionListener scanListener = new ActionListener()
      {
         @Override
         public void actionPerformed(final ActionEvent e)
         {
            scanSelectionChanged();
         }
      };
      
      String localNet = Util.getIntIp();
      localNet = localNet.substring(0, localNet.lastIndexOf('.') + 1) + '*';
      
      scanLocalNet.addActionListener(scanListener);
      scanSpecificIps.addActionListener(scanListener);

      scanLocalNet.setBorder(new EmptyBorder(0, 0, 5, 20));      
      add(scanLocalNet, 0, y, GridBagConstraints.NORTHWEST);
      
      add(new JLabel("<html>This option will automatically scan the local network (" + localNet 
                     + ")<br /> for other computers running " + Voip2p.APP_NAME + '.'), 1, y++, GridBagConstraints.WEST);

      add(new JLabel(" "), 0, y++);
      
      add(scanSpecificIps, 0, y, GridBagConstraints.NORTHWEST, 1, 2);
      
      add(new JLabel("<html>This option will only scan the given IP addresses.<br />"
            + "Enter in the IP addresses to scan, separated with spaces or line breaks.</html>"), 1, y++, GridBagConstraints.WEST);
      
      specificIps.setColumns(40);
      specificIps.setRows(5);
      specificIps.setWrapStyleWord(true);
      add(new JScrollPane(specificIps), 1, y++);
      
      add(new JLabel(" "), 0, y++);
      
      add(new JLabel("Application data TCP Port: "), 0, y, GridBagConstraints.WEST);
      add(tcpPort, 1, y++, GridBagConstraints.WEST);
      
      add(new JLabel("Voice comms UDP Port: "), 0, y, GridBagConstraints.WEST);
      add(udpPort, 1, y++, GridBagConstraints.WEST);      
   }
   
   private void scanSelectionChanged()
   {
      specificIps.setEnabled(scanSpecificIps.isSelected());
   }
   
   /** {@inheritDoc} */
   @Override
   public void doValidate(final List<ValidationError> errors)
   {
      if (tcpPort.getIntValue() < 1 || tcpPort.getIntValue() > 65535)
      {
         errors.add(new ValidationError("The TCP port must be between 1 and 65535", tcpPort));
      }
      
      if (udpPort.getIntValue() < 1 || udpPort.getIntValue() > 65535)
      {
         errors.add(new ValidationError("The UDP port must be between 1 and 65535", udpPort));
      }
      
      if (scanSpecificIps.isSelected())
      {
         StringTokenizer st = new StringTokenizer(specificIps.getText());
         
         if (!st.hasMoreTokens())
         {
            errors.add(new ValidationError("At least one specific IP address must be entered", specificIps));
         }
         else
         {
            while (st.hasMoreTokens())
            {
               String ip = st.nextToken();
               boolean valid = Pattern.matches("([0-9]{1,3}\\.){3}[0-9]{1,3}", ip);
 
               if (valid)
               {
                  try
                  {
                     InetAddress.getByName(ip);
                  }
                  catch (Exception e)
                  {
                     valid = false;
                  }
               }
               
               if (!valid)
               {
                  errors.add(new ValidationError("The address " + ip + " is invalid", specificIps));
                  break;
               }
            }
         }
      }
   }
   
   /** {@inheritDoc} */
   @Override
   public void reset()
   {
      udpPort.setIntValue(AppConfig.getUdpPort());
      tcpPort.setIntValue(AppConfig.getTcpPort());
      
      scanLocalNet.setSelected(AppConfig.isScanLocalNetwork());
      
      scanSpecificIps.setSelected(!scanLocalNet.isSelected());
      specificIps.setEnabled(scanSpecificIps.isSelected());
      
      StringBuilder buf = new StringBuilder();
      
      for (String ip : AppConfig.getIpsToScan())
      {
         if (buf.length() > 0)
         {
            buf.append('\t');
         }
         
         buf.append(ip);
      }
      
      specificIps.setText(buf.toString());
   }
   
   /** {@inheritDoc} */
   @Override
   public void save()
   {
      AppConfig.setUdpPort(udpPort.getIntValue());
      AppConfig.setTcpPort(tcpPort.getIntValue());
      AppConfig.setScanLocalNetwork(scanLocalNet.isSelected());
      
      if (scanSpecificIps.isSelected())
      {
         AppConfig.setIpsToScan(specificIps.getText().split("[ \t\n\r\f]+"));
      }
   }
}
