package com.aleax.voip.ui.settings;

import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.aleax.voip.ui.settings.audio.AudioSettingsPanel;
import com.aleax.voip.util.Util;
import com.aleax.voip.util.XPanel;

/**
 * The dialog contains all of the settings panels for the application.
 * 
 * @author Yiannis Paschalidis
 */
public class SettingsDialog extends JDialog implements ChangeListener
{
   /** The tabs containing the settings panels. */
   private final JTabbedPane tabs = new JTabbedPane();
   
   /** General application settings. */
   private final GeneralSettingsPanel generalSettings = new GeneralSettingsPanel();
   
   /** Audio settings. */
   private final AudioSettingsPanel audioSettings = new AudioSettingsPanel();
   
   /** Network settings. */
   private final NetworkSettingsPanel networkSettings = new NetworkSettingsPanel();
   
   /** The collection of panels for convenient iteration. */
   private final AbstractSettingsPanel[] panels = { generalSettings, audioSettings, networkSettings };
   
   /** Accepts changes and closes the dialog. */
   private final JButton okButton = new JButton("Ok");
   
   /** Discards changes and closes the dialog. */
   private final JButton cancelButton = new JButton("Cancel");
   
   /** The owner of this dialog. */
   private final Component owner;
   
   /**
    * Creates a SettingsDialog.
    * @param owner the owning component.
    */
   public SettingsDialog(final Component owner)
   {
      super(Util.getParentWindow(owner), "Settings");
      
      this.owner = owner;
      
      XPanel contentPane = new XPanel();
      contentPane.setBorder(new EmptyBorder(5, 0, 5, 0));
      setContentPane(contentPane);
      
      Border border = new EmptyBorder(10, 10, 0, 10);
      generalSettings.setBorder(border);
      audioSettings.setBorder(border);
      networkSettings.setBorder(border);
      
      tabs.add("General", generalSettings);
      tabs.add("Audio", audioSettings);
      tabs.add("Network", networkSettings);
      contentPane.addFillXY(tabs, 0, 0);
      
      okButton.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(final ActionEvent e)
         {
            for (AbstractSettingsPanel panel : panels)
            {
               panel.save();
            }
            
            setVisible(false);
         }
      });
      
      cancelButton.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(final ActionEvent e)
         {
            setVisible(false);
         }
      });
      
      JPanel buttonPanel = new JPanel();
      buttonPanel.setLayout(new FlowLayout(FlowLayout.RIGHT, 5, 0));
      buttonPanel.add(okButton);
      buttonPanel.add(cancelButton);
      contentPane.addFillX(buttonPanel, 0, 1);
      
      for (AbstractSettingsPanel panel : panels)
      {
         panel.setController(this);
      }
   }
   
   /** {@inheritDoc} */
   @Override
   public void stateChanged(final ChangeEvent e)
   {
      for (AbstractSettingsPanel panel : panels)
      {
         if (!panel.validateInput())
         {
            okButton.setEnabled(false);
            return;
         }
      }
      
      okButton.setEnabled(true);
   }
 
   /**
    * Displays the dialog.
    */
   public void display()
   {
      // Reset each panel based on the current config.
      for (AbstractSettingsPanel panel : panels)
      {
         panel.reset();
      }

      // Force validation and update button state.
      stateChanged(null);
      
      pack();
      Util.centerWindow(this, owner);
      setVisible(true);
   }
}
