package com.aleax.voip.ui.settings;

import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.List;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.border.Border;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.aleax.voip.ui.AppConfig;
import com.aleax.voip.ui.Voip2p;
import com.aleax.voip.ui.settings.audio.AudioSettingsPanel;
import com.aleax.voip.util.Util;
import com.aleax.voip.util.ValidationError;
import com.aleax.voip.util.XPanel;

/**
 * The settings wizard dialog provides step-by-step configuration for first-time users.
 * 
 * @author Yiannis Paschalidis
 */
public class SettingsWizardDialog extends JDialog implements ChangeListener
{
   /** Welcome panel. */
   private final WelcomePanel welcomePanel = new WelcomePanel();
   
   /** General application settings. */
   private final GeneralSettingsPanel generalSettings = new GeneralSettingsPanel();
   
   /** Audio settings. */
   private final AudioSettingsPanel audioSettings = new AudioSettingsPanel();
   
   /** Network settings. */
   private final NetworkSettingsPanel networkSettings = new NetworkSettingsPanel();
   
   /** The collection of panels for convenient iteration. */
   private final AbstractSettingsPanel[] panels = { welcomePanel, generalSettings, audioSettings, networkSettings };

   /** Prompts for each panel. */
   private final String[] prompts = 
   {
      null, 
      
      "<b>Step 1/3 - General settings</b>\nEnter the name that other users will see you as.", 
      "<b>Step 2/3 - Audio settings</b>\nSelect the audio devices for recording and playback, as well as sound quality.\n<i>Note:</i> Higher quality sound uses more network bandwidth.", 
      "<b>Step 3/3 - Network settings</b>\nIf you are running " + Voip2p.APP_NAME + " on a home network, the default settings should work fine.\nIf you are running the application over the internet, you should enter specific IP addresses,\nand you may need to change the port setings and set up port-forwarding on your router." 
   };
   
   /** Accepts changes and closes the dialog. */
   private final JButton finishButton = new JButton("Finish");
   
   /** Discards changes and closes the dialog. */
   private final JButton cancelButton = new JButton("Cancel");

   /** Displays the previous wizard step. */
   private final JButton previousButton = new JButton("Previous");
   
   /** Displays the next wizard step. */
   private final JButton nextButton = new JButton("Next");
   
   /** The owner of this dialog. */
   private final Component owner;
   
   /** The current wizard index. */
   private int index = 0;

   /** The content pane which displays the setting panel. */
   private final XPanel settingPanel = new XPanel();
   
   /** Indicates whether the settings wizard has been completed. */
   private boolean completed = false;

   /**
    * Creates a SettingsDialog.
    * @param owner the owning component.
    */
   public SettingsWizardDialog(final Component owner)
   {
      super(Util.getParentWindow(owner), Voip2p.APP_NAME + " first time configuration");
      
      this.owner = owner;
      
      XPanel contentPane = new XPanel();
      contentPane.setBorder(new EmptyBorder(5, 5, 5, 5));
      setContentPane(contentPane);
      contentPane.addFillXY(settingPanel, 0, 0);
      
      Border border = new CompoundBorder(new EtchedBorder(EtchedBorder.LOWERED), new EmptyBorder(10, 10, 10, 10));
      generalSettings.setBorder(border);
      audioSettings.setBorder(border);
      networkSettings.setBorder(border);
      
      finishButton.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(final ActionEvent e)
         {
            completed = true;
            
            for (AbstractSettingsPanel panel : panels)
            {
               panel.save();
            }
            
            setVisible(false);
            AppConfig.save();
            new Voip2p();
         }
      });
      
      cancelButton.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(final ActionEvent e)
         {
            confirmExit();
         }
      });
      
      nextButton.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(final ActionEvent e)
         {
            displayPanel(Math.min(panels.length - 1, index + 1));
         }
      });
      
      previousButton.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(final ActionEvent e)
         {
            displayPanel(Math.max(0, index - 1));
         }
      });
      
      JPanel buttonPanel = new JPanel();
      buttonPanel.setBorder(new EmptyBorder(10, 0, 0, 0));
      buttonPanel.setLayout(new FlowLayout(FlowLayout.RIGHT, 5, 0));
      buttonPanel.add(previousButton);
      buttonPanel.add(nextButton);
      buttonPanel.add(finishButton);
      buttonPanel.add(cancelButton);
      contentPane.addFillX(buttonPanel, 0, 1);
      
      for (AbstractSettingsPanel panel : panels)
      {
         panel.setController(this);
      }
      
      setResizable(false);
      displayPanel(0);

      setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
      addWindowListener(new WindowAdapter()      
      {
         @Override
         public void windowClosing(final WindowEvent e)
         {
            if (!completed)
            {
               confirmExit();
            }
         }
      });
   }
   
   /**
    * Allows the user to confirm that they want to exit the application.
    */
   private void confirmExit()
   {
      if (JOptionPane.showConfirmDialog(SettingsWizardDialog.this, "Set-up is not complete. Do you want to exit?", 
            "Confirm exit", JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION)
      {
         setVisible(false);
         System.exit(0);
      }
   }
   
   /** {@inheritDoc} */
   @Override
   public void stateChanged(final ChangeEvent e)
   {
      boolean valid = panels[index].validateInput(); 
      nextButton.setEnabled(valid);
      finishButton.setEnabled(valid);
   }
 
   /**
    * Displays the dialog.
    */
   public void display()
   {
      for (AbstractSettingsPanel panel : panels)
      {
         panel.reset();
      }
      
      pack();
      Util.centerWindow(this, owner);
      setVisible(true);
   }

   /**
    * Displays the settings panel with the given index.
    * @param index the index of the panel to display.
    */
   private void displayPanel(final int index)
   {
      this.index = index;
      boolean last = index == panels.length - 1;
      boolean valid = panels[index].validateInput();
      
      nextButton.setVisible(!last);
      nextButton.setEnabled(valid);
      finishButton.setVisible(last);
      finishButton.setEnabled(valid);
      previousButton.setEnabled(index > 0);
      
      settingPanel.removeAll();
      
      if (prompts[index] != null)
      {
         JLabel label = new JLabel("<html>" + prompts[index].replaceAll("\n", "<br />") + "</html>");
         label.setBorder(new EmptyBorder(5, 0, 10, 0));
         
         settingPanel.addFillX(label, 0, 0);
      }
      
      settingPanel.addFillXY(panels[index], 0, 1);
      pack();
      Util.centerWindow(this, owner);
   }

   /**
    * Displays an application welcome message. 
   */
   private static final class WelcomePanel extends AbstractSettingsPanel
   {
      /**
       * Creates a WelcomePanel.
       */
      public WelcomePanel()
      {
         setBorder(new EmptyBorder(10, 10, 10, 10));
         
         ImageIcon image = new ImageIcon(Util.getImage("/images/voip2p_logo_128_trans.png"));
         
         String text = "<html><h2>Voip2p</h2>Simple peer-to-peer voice chat for your LAN.<br />"
                     + "Before you get started, there are a few items which need to be set up.<br />"
                     + "These can be changed later via \"Settings\" in the main menu.<br /><br />"
                     + "</html>";
         
         add(new JLabel(image), 0, 0, GridBagConstraints.CENTER);
         add(new JLabel(text), 0, 1, GridBagConstraints.HORIZONTAL, 1, 1, GridBagConstraints.NORTH);
      }
      
      /** {@inheritDoc} */
      @Override
      public void doValidate(final List<ValidationError> errors)
      {
         // NOP
      }
      
      /** {@inheritDoc} */
      @Override
      public void reset()
      {
         // NOP
      }
      
      /** {@inheritDoc} */
      @Override
      public void save()
      {
         // NOP
      }
   }
}
