package com.aleax.voip.ui.settings.audio;

import java.awt.GridBagConstraints;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.Line;
import javax.sound.sampled.Mixer;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;

import com.aleax.voip.util.AudioUtil;
import com.aleax.voip.util.LayeredBorder;
import com.aleax.voip.util.XPanel;

/**
 * Allows the user to select the audio format for a single device.
 * @author Yiannis Paschalidis
 */
public final class AudioFormatSelectionPanel extends XPanel
{
   /** The default sample rates to present when the device does not explicitly provide any. */
   private static final int[] DEFAULT_SAMPLE_RATES = { 8000, 16000, 22050, 44100, 48000, 96000 };
   
   /** Allows the user to specify the number of channels. */
   private final JComboBox<ChannelOption> channelSelector = new JComboBox<ChannelOption>();
   
   /** Allows the user to specify the bit depth. */
   private final JComboBox<BitSizeOption> bitSizeSelector = new JComboBox<BitSizeOption>();
   
   /** Allows the user to specify the sample rate. */
   private final JComboBox<SampleRateOption> sampleRateSelector = new JComboBox<SampleRateOption>();
   
   /** The complete list of supported formats for the device. */
   private final List<AudioFormat> supportedFormats = new ArrayList<AudioFormat>(); 
   
   /**
    * Creates an AudioFormatSelectionPanel.
    */
   public AudioFormatSelectionPanel()
   {
      setBorder(new LayeredBorder(new EmptyBorder(5, 5, 15, 5), 
                                  new EtchedBorder(EtchedBorder.LOWERED), 
                                  new EmptyBorder(5, 5, 5, 5)));
      
      add(new JLabel("Channels: "), 0, 0, GridBagConstraints.WEST);
      add(channelSelector, 1, 0, GridBagConstraints.WEST);
      
      add(new JLabel("Bit depth: "), 0, 1, GridBagConstraints.WEST);
      add(bitSizeSelector, 1, 1, GridBagConstraints.WEST);
      
      add(new JLabel("Sample rate: "), 0, 2, GridBagConstraints.WEST);
      add(sampleRateSelector, 1, 2, GridBagConstraints.WEST);
      
      channelSelector.addItemListener(new ItemListener()
      {
         @Override
         public void itemStateChanged(final ItemEvent e)
         {
            channelSelectionChanged();
         }
      });
      
      bitSizeSelector.addItemListener(new ItemListener()
      {
         @Override
         public void itemStateChanged(final ItemEvent e)
         {
            bitSizeChanged();
         }
      });
      
      sampleRateSelector.addItemListener(new ItemListener()
      {
         @Override
         public void itemStateChanged(final ItemEvent e)
         {
            sampleRateChanged();
         }
      });
   }

   /**
    * Resets the panel to obtain settings for the given device.
    * @param mixer the device mixer.
    * @param lines the device lines.
    */
   public void reset(final Mixer.Info mixer, final Line.Info[] lines)
   {
      supportedFormats.clear();
      
      for (Line.Info line : lines)
      {
         if (line instanceof DataLine.Info)
         {
            supportedFormats.addAll(Arrays.asList(((DataLine.Info) line).getFormats()));
         }
      }
      
      deviceSelectionChanged();
   }
   
   /**
    * Called internally when the device changes.
    * Updates all fields according to the currently available formats.
    */
   private void deviceSelectionChanged()
   {
      ChannelOption currentChannel = (ChannelOption) channelSelector.getSelectedItem();
      
      final Set<ChannelOption> optionSet = new HashSet<ChannelOption>();
      
      for (AudioFormat format : supportedFormats)
      {
         int n = format.getChannels();

         if (n == AudioSystem.NOT_SPECIFIED)
         {
            // Any number of channels supported - just add mono/stereo
            optionSet.add(new ChannelOption(1));
            optionSet.add(new ChannelOption(2));
         }
         else
         {
            optionSet.add(new ChannelOption(n));
         }
      }
      
      ChannelOption[] sortedOptions = optionSet.toArray(new ChannelOption[optionSet.size()]);
      Arrays.sort(sortedOptions);
      
      if (currentChannel == null && sortedOptions.length > 0)
      {
         currentChannel = sortedOptions[0];
      }
      
      channelSelector.setModel(new DefaultComboBoxModel<ChannelOption>(sortedOptions));
      channelSelector.setSelectedItem(currentChannel);
      channelSelectionChanged();
   }

   /**
    * Called internally when the channel setting is changed.
    * Updates remaining fields according to the currently available formats.
    */
   private void channelSelectionChanged()
   {
      BitSizeOption currentBitSize = (BitSizeOption) bitSizeSelector.getSelectedItem();
      final Set<BitSizeOption> optionSet = new HashSet<BitSizeOption>();

      List<AudioFormat> filteredFormats = new ArrayList<AudioFormat>(supportedFormats);
      filterFormats(filteredFormats, (ChannelOption) channelSelector.getSelectedItem(), null, null);
      
      for (AudioFormat format : filteredFormats)
      {
         int n = format.getSampleSizeInBits();
       
         if (n == AudioSystem.NOT_SPECIFIED)
         {
            // Any size is supported - just add 8-bit / 16-bit
            optionSet.add(new BitSizeOption(8));
            optionSet.add(new BitSizeOption(16));
         }
         else
         {
            optionSet.add(new BitSizeOption(n));
         }
      }
      
      BitSizeOption[] sortedOptions = optionSet.toArray(new BitSizeOption[optionSet.size()]);
      Arrays.sort(sortedOptions);
      
      if (currentBitSize == null && sortedOptions.length > 0)
      {
         currentBitSize = sortedOptions[0];
      }
      
      bitSizeSelector.setModel(new DefaultComboBoxModel<BitSizeOption>(sortedOptions));
      bitSizeSelector.setSelectedItem(currentBitSize);
      bitSizeChanged();
   }
   
   /**
    * Called internally when the bit depth setting is changed.
    * Updates remaining fields according to the currently available formats.
    */
   private void bitSizeChanged()
   {
      SampleRateOption currentSampleRate = (SampleRateOption) sampleRateSelector.getSelectedItem();
      final Set<SampleRateOption> optionSet = new HashSet<SampleRateOption>();

      List<AudioFormat> filteredFormats = new ArrayList<AudioFormat>(supportedFormats);
      filterFormats(filteredFormats, (ChannelOption) channelSelector.getSelectedItem(), (BitSizeOption) bitSizeSelector.getSelectedItem(), null);
      
      for (AudioFormat format : filteredFormats)
      {
         int n = (int) format.getSampleRate();
       
         if (n == AudioSystem.NOT_SPECIFIED)
         {
            // Any rate is supported - add most common
            for (int defaultRate : DEFAULT_SAMPLE_RATES)
            {
               optionSet.add(new SampleRateOption(defaultRate));
            }
         }
         else
         {
            optionSet.add(new SampleRateOption(n));
         }
      }
      
      SampleRateOption[] sortedOptions = optionSet.toArray(new SampleRateOption[optionSet.size()]);
      Arrays.sort(sortedOptions);
      
      if (currentSampleRate == null && sortedOptions.length > 0)
      {
         currentSampleRate = sortedOptions[0];
      }
      
      sampleRateSelector.setModel(new DefaultComboBoxModel<SampleRateOption>(sortedOptions));
      sampleRateSelector.setSelectedItem(currentSampleRate);
      sampleRateChanged();
   }
   
   /**
    * Called internally when the sample rate is changed.
    */
   private void sampleRateChanged()
   {
      // Do nothing
   }
   
   /**
    * @return the currently selected audio format.
    */
   public AudioFormat getFormat()
   {
      int channels = ((ChannelOption) channelSelector.getSelectedItem()).getChannels();
      int bitSize = ((BitSizeOption) bitSizeSelector.getSelectedItem()).getBits();
      int sampleRate = ((SampleRateOption) sampleRateSelector.getSelectedItem()).getRate();
      int frameRate = sampleRate; // TODO: Only for uncompressed
            
      List<AudioFormat> filteredFormats = new ArrayList<AudioFormat>(supportedFormats);
      AudioUtil.filterFormats(filteredFormats, channels, bitSize, sampleRate);
      
      AudioFormat format = filteredFormats.isEmpty() ? null : filteredFormats.get(0);
      
      if (format != null)
      {
         format = new AudioFormat(format.getEncoding(), sampleRate, bitSize, channels, format.getFrameSize(), frameRate, format.isBigEndian(), format.properties());
      }
      
      return format;
   }

   /**
    * Filters the given formats based on the user's selections, or a partial selection.
    * 
    * @param allFormats the formats to filter.
    * @param channels the channel selection.
    * @param bitSize the bit size selection.
    * @param sampleRate the sample rate selection.
    */
   private static void filterFormats(final List<AudioFormat> allFormats, final ChannelOption channels, final BitSizeOption bitSize, final SampleRateOption sampleRate)
   {
      AudioUtil.filterFormats(allFormats, channels == null ? AudioSystem.NOT_SPECIFIED : channels.getChannels(),
                                bitSize == null ? AudioSystem.NOT_SPECIFIED : bitSize.getBits(),
                                sampleRate == null ? AudioSystem.NOT_SPECIFIED : sampleRate.getRate());
   }
}
