package com.aleax.voip.ui.settings.audio;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.List;
import java.util.Map;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.Line;
import javax.sound.sampled.Mixer;
import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JLabel;

import com.aleax.voip.ui.AppConfig;
import com.aleax.voip.ui.settings.AbstractSettingsPanel;
import com.aleax.voip.util.AudioUtil;
import com.aleax.voip.util.Dual;
import com.aleax.voip.util.ValidationError;

/**
 * Allows the user to set up recording and playback devices and formats.
 * @author Yiannis Paschalidis
 */
public final class AudioSettingsPanel extends AbstractSettingsPanel
{
   /** The available recording mixers. */
   private Map<Mixer.Info, Line.Info[]> recordingMixers;
   
   /** The available playback mixers. */
   private Map<Mixer.Info, Line.Info[]> playbackMixers;
   
   /** Used to select the recording device to use. */
   private final JComboBox<Mixer.Info> recordingDeviceSelector = new JComboBox<Mixer.Info>();
   
   /** Used to select the playback device to use. */
   private final JComboBox<Mixer.Info> playbackDeviceSelector = new JComboBox<Mixer.Info>();
   
   /** Used to select the recording format. */
   private final AudioFormatSelectionPanel recordingFormatPanel = new AudioFormatSelectionPanel();
   
   /** Used to select the playback format. */
   private final AudioFormatSelectionPanel playbackFormatPanel = new AudioFormatSelectionPanel();

   /**
    * Creates an AudioSettingsPanel.
    */
   public AudioSettingsPanel()
   {
      addFillX(new JLabel("Recording device: "), 0, 0);
      addFillX(recordingDeviceSelector, 1, 0);
      addFillXY(recordingFormatPanel, 0, 1, 2, 1);
      
      addFillX(new JLabel("Playback device: "), 0, 2);
      addFillX(playbackDeviceSelector, 1, 2);
      addFillXY(playbackFormatPanel, 0, 3, 2, 1);
      
      reset();
      
      playbackDeviceSelector.addItemListener(new ItemListener()
      {
         @Override
         public void itemStateChanged(final ItemEvent e)
         {
            Mixer.Info device = (Mixer.Info) playbackDeviceSelector.getSelectedItem();
            playbackFormatPanel.reset(device, playbackMixers.get(device));
         }
      });
      
      recordingDeviceSelector.addItemListener(new ItemListener()
      {
         @Override
         public void itemStateChanged(final ItemEvent e)
         {
            Mixer.Info device = (Mixer.Info) recordingDeviceSelector.getSelectedItem();
            recordingFormatPanel.reset(device, recordingMixers.get(device));
         }
      });
   }
   
   /** {@inheritDoc} */
   @Override
   public void reset()
   {
      // TODO: use existing config.
      Dual<Mixer.Info, AudioFormat> recordingDevice = AppConfig.getRecordingDevice();
      Dual<Mixer.Info, AudioFormat> playbackDevice = AppConfig.getPlaybackDevice();
      
      recordingMixers = AudioUtil.getRecordingDevices();
      playbackMixers = AudioUtil.getPlaybackDevices();
      
      ComboBoxModel<Mixer.Info> model = new DefaultComboBoxModel<>(recordingMixers.keySet().toArray(new Mixer.Info[recordingMixers.size()]));
      recordingDeviceSelector.setModel(model);
      
      model = new DefaultComboBoxModel<>(playbackMixers.keySet().toArray(new Mixer.Info[playbackMixers.size()]));
      playbackDeviceSelector.setModel(model);
     
      Mixer.Info device = (Mixer.Info) recordingDeviceSelector.getSelectedItem();
      recordingFormatPanel.reset(device, recordingMixers.get(device));
      
      device = (Mixer.Info) playbackDeviceSelector.getSelectedItem();
      playbackFormatPanel.reset(device, playbackMixers.get(device));
   }

   /** {@inheritDoc} */
   @Override
   public void save()
   {
      AppConfig.setRecordingDevice(getRecordingDevice());
      AppConfig.setPlaybackDevice(getPlaybackDevice());
   }
   
   /** {@inheritDoc} */
   @Override
   public void doValidate(final List<ValidationError> errors)
   {
      // Nothing to validate - the user can only select valid items.
   }
   
   /** @return the user's selected recording device and format. */
   public Dual<Mixer.Info, AudioFormat> getRecordingDevice()
   {
      return new Dual<Mixer.Info, AudioFormat>((Mixer.Info) recordingDeviceSelector.getSelectedItem(), recordingFormatPanel.getFormat());
   }
   
   /** @return the user's selected playback device and format. */
   public Dual<Mixer.Info, AudioFormat> getPlaybackDevice()
   {
      return new Dual<Mixer.Info, AudioFormat>((Mixer.Info) playbackDeviceSelector.getSelectedItem(), playbackFormatPanel.getFormat());
   }
}