package com.aleax.voip.util;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioFormat.Encoding;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Line;
import javax.sound.sampled.Mixer;
import javax.sound.sampled.SourceDataLine;
import javax.sound.sampled.TargetDataLine;

/**
 * Audio related utility methods.
 * 
 * @author Yiannis Paschalidis
 */
public final class AudioUtil
{
   /** The logger instance for this class. */
   private static final Logger log = LogUtil.get(AudioUtil.class);
   
   /** Delimter used for serializing device settings. */
   private static final char DELIMITER = '\u0007';
   
   /** Delimter used for serializing device settings. */
   private static final String DELIMITER_STR = "\u0007"; 
   
   /** Prevent instantiation of this utility class. */
   private AudioUtil()
   {
   }
   
   /**
    * @return a map of available audio recording mixers and the lines they support.
    */
   public static Map<Mixer.Info, Line.Info[]> getRecordingDevices()
   {
      final Map<Mixer.Info, Line.Info[]> recordingMixers = new HashMap<Mixer.Info, Line.Info[]>();
      Line.Info targetDLInfo = new Line.Info(TargetDataLine.class);
      
      for (Mixer.Info mixerInfo : AudioSystem.getMixerInfo())
      {
         Mixer temp = AudioSystem.getMixer(mixerInfo);

         if (temp.isLineSupported(targetDLInfo))
         {
            recordingMixers.put(mixerInfo, temp.getTargetLineInfo());
         }
      }

      return recordingMixers;
   }

   /**
    * @return a map of available audio playback mixers and the lines they support.
    */
   public static Map<Mixer.Info, Line.Info[]> getPlaybackDevices()
   {
      final Map<Mixer.Info, Line.Info[]> playbackMixers = new HashMap<Mixer.Info, Line.Info[]>();
      Line.Info sourceDLInfo = new Line.Info(SourceDataLine.class);

      for (Mixer.Info mixerInfo : AudioSystem.getMixerInfo())
      {
         Mixer temp = AudioSystem.getMixer(mixerInfo);

         if (temp.isLineSupported(sourceDLInfo))
         {
            playbackMixers.put(mixerInfo, temp.getSourceLineInfo());
         }
      }

      return playbackMixers;
   }

   /**
    * Removes unsupported audio formats from the list.
    * 
    * @param allFormats the list of formats to modify.
    * @param channels the number of desired channels.
    * @param bitSize the desired bit depth.
    * @param sampleRate the desired sample rate.
    */
   public static void filterFormats(final List<AudioFormat> allFormats, final int channels, final int bitSize, final int sampleRate)
   {
      for (Iterator<AudioFormat> i = allFormats.iterator() ; i.hasNext() ;)
      {
         AudioFormat format = i.next();
         
         if (channels != AudioSystem.NOT_SPECIFIED)
         {
            int n = format.getChannels();
            
            if (n != AudioSystem.NOT_SPECIFIED && n != channels)
            {
               i.remove();
               continue;
            }
         }
          
         if (bitSize != AudioSystem.NOT_SPECIFIED)
         {
            int n = format.getSampleSizeInBits();
            
            if (n != AudioSystem.NOT_SPECIFIED && n != bitSize)
            {
               i.remove();
               continue;
            }
         }
         
         if (sampleRate != AudioSystem.NOT_SPECIFIED)
         {
            int n = (int) format.getSampleRate();
            
            if (n != AudioSystem.NOT_SPECIFIED && n != sampleRate)
            {
               i.remove();
               continue;
            }
         }
      }
   }
   
   public static Dual<Mixer.Info, AudioFormat> decodeAudioDevice(final String encoded)
   {
      try
      {
         if (encoded != null)
         {
            int index = -1;
            
            for (int i = 0 ; i < 3 ; i++)
            {
               index = encoded.indexOf(DELIMITER, index + 1);
            }

            if (index > 0)
            {
               String[] parts = encoded.substring(0, index).split(DELIMITER_STR);
               AudioFormat format = decodeAudioFormat(encoded.substring(index + 1));
            
               boolean input = "in".equals(parts[0]);
               String name = parts[1];
               String vendor = parts[2];
               
               Map<Mixer.Info, Line.Info[]> devices = input ? getRecordingDevices() : getPlaybackDevices();
               
               for (Map.Entry<Mixer.Info, Line.Info[]> device : devices.entrySet())
               {
                  if (name.equals(device.getKey().getName()) && vendor.equals(device.getKey().getVendor())) 
                  {                     
                     return new Dual<Mixer.Info, AudioFormat>(device.getKey(), format);
                  }
               }
            }            
         }
      }
      catch (Exception e)
      {
         log.log(Level.WARNING, "Invalid audio setting: " + encoded, e);
      }
      
      return null;
   }

   public static String encodeAudioDevice(final Dual<Mixer.Info, AudioFormat> device, final boolean inputDevice)
   {
      if (device != null && device.getFirst() != null && device.getSecond() != null)
      {
         return (inputDevice ? "in" : "out")  
               + DELIMITER + device.getFirst().getName() 
               + DELIMITER + device.getFirst().getVendor()
               + DELIMITER + encodeAudioFormat(device.getSecond());
      }

      return null;
   }
   
   public static String encodeAudioFormat(final AudioFormat format)
   {
      if (format != null)
      {
         return format.getEncoding().toString()
               + DELIMITER + format.getChannels()
               + DELIMITER + format.getSampleSizeInBits()
               + DELIMITER + (int) format.getSampleRate()
               + DELIMITER + format.getFrameSize()
               + DELIMITER + (int) format.getFrameRate()
               + DELIMITER + format.isBigEndian();
      }
      
      return null;
   }
   
   public static AudioFormat decodeAudioFormat(final String format)
   {
      if (format != null)
      {
         try
         {
            String[] parts = format.split(DELIMITER_STR);
            
            if (parts.length == 7)
            {
               Encoding encoding = new Encoding(parts[0]);
               int channels = Integer.parseInt(parts[1]);
               int sampleSizeInBits = Integer.parseInt(parts[2]);
               int sampleRate = Integer.parseInt(parts[3]);
               int frameSize = Integer.parseInt(parts[4]);
               int frameRate = Integer.parseInt(parts[5]);
               boolean bigEndian = "true".equals(parts[6]);
               
               return new AudioFormat(encoding, sampleRate, sampleSizeInBits, channels, frameSize, frameRate, bigEndian);
            }
         }
         catch (Exception e)
         {
            log.log(Level.WARNING, "Invalid audio format: " + format, e);
         }
      }
      
      return null;
   }
}
