package com.aleax.voip.util;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;

import javax.swing.JScrollPane;
import javax.swing.JViewport;
import javax.swing.SwingUtilities;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * AutoScrollPane - extends JScrollPane to scroll automatically when content is added.
 *
 * @version 1.0 27 Mar 99
 * @author Yiannis Paschalidis
 */
public final class AutoScrollPane extends JScrollPane implements ChangeListener
{
   private int oldSize = Integer.MIN_VALUE;
   private boolean atEnd = true;
   private final Point point = new Point(0, 0);

   /**
    * Creates an AutoScrollPane.
    * @param c the component to scroll.
    */
   public AutoScrollPane(final Component c)
   {
      super(c);
      getViewport().addChangeListener(this);
   }

   /**
    * Creates an AutoScrollPane.
    * @param c the component to scroll.
    * @param vsbPolicy the vertical scrollbar policy.
    * @param hsbPolicy the vertical scrollbar policy.
    */
   public AutoScrollPane(final Component c, final int vsbPolicy, final int hsbPolicy)
   {
      super(c,vsbPolicy,hsbPolicy);
      getViewport().addChangeListener(this);
   }

   /** {@inheritDoc} */
   @Override
   public void stateChanged(final ChangeEvent e)
   {
      JViewport vp = getViewport();
      int h = vp.getViewSize().height;
    
      // This event is also fired when the user moves the scrollbar, 
      // so check if content has been added
    
      if (h != oldSize) 
      {
         //If the user has scrolled back to look at previous content,
         //don't scroll it down to the bottom.

         if (atEnd)
         {
            point.x = 0;
            point.y = h;
            vp.setViewPosition(point);
         }

         oldSize = h;
      }
      else
      {
         atEnd = (vp.getViewPosition().y + vp.getExtentSize().height >= vp.getView().getHeight());
      }
   }

   /** {@inheritDoc} */
   @Override
   public void setSize(final Dimension d)
   {
      // Overriden to scroll content to its end on resize.
      super.setSize(d);
      scrollEndLater();
   }

   /** {@inheritDoc} */
   @Override
   public void setSize(final int width, final int height)
   {
      // Overriden to scroll content to its end on resize.
      super.setSize(width,height);
      scrollEndLater();
   }

   /** {@inheritDoc} */
   @Override
   public void setBounds(final Rectangle r)
   {
      // Overriden to scroll content to its end on resize.
      super.setBounds(r);
      scrollEndLater();
   }

   /** {@inheritDoc} */
   @Override
   public void setBounds(final int x, final int y, final int width, final int height)
   {
      // Overriden to scroll content to its end on resize.
      super.setBounds(x,y,width,height);
      scrollEndLater();
   }

   /**
    * Scrolls the contents to the top.
    */
   public void scrollHome()
   {
      JViewport vp = getViewport();
      vp.setViewPosition(new Point(0, 0));
   }

   /**
    * Scrolls the contents to the bottom.
    */
   public void scrollEnd()
   {
      JViewport vp = getViewport();
      int dy = vp.getExtentSize().height;

      Point p = new Point(0, vp.getView().getHeight() - dy);
      vp.setViewPosition(p);
   }

   /**
    * Scrolls the contents up/down <CODE>delta</CODE> pages.
    */
   public void scrollPages(final int delta)
   {
      JViewport vp = getViewport();

      int dy = vp.getExtentSize().height;

      Point p = vp.getViewPosition();
      p.y += dy * delta;

      p.y = Math.min(Math.max(p.y, 0), vp.getView().getHeight() - dy);

      vp.setViewPosition(p);
   }

   /**
    * Scrolls to the end later in the Swing thread.
    */
   private void scrollEndLater()
   {
      SwingUtilities.invokeLater
      (
         new Runnable()
         {
            @Override
            public void run()
            {
               scrollEnd();
            }
         }
      );
   }
}
