package com.aleax.voip.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.Serializable;
import java.util.Collections;
import java.util.Map;
import java.util.Properties;

/**
 * The Config class provides convenient storage for program configuration.
 * 
 * @author Yiannis Paschalidis
 */
public final class Config implements Serializable
{
   /** The backing properties instance. */
   private Properties backing = new Properties();

   /**
    * Removes the configuration parameter with the given key.
    * @param key the key of the parameter to remove.
    */
   public void unset(final String key)
   {
      backing.remove(key);
   }

   /**
    * Removes all configuration parameters.
    */
   public void clear()
   {
      backing.clear();
   }
   
   /**
    * Sets a configuration parameter.
    * 
    * @param key the key.
    * @param value the value;
    */
   public void set(final String key, final int value)
   {
      backing.setProperty(key, String.valueOf(value));
   }

   /**
    * Sets a configuration parameter.
    * 
    * @param key the key.
    * @param value the value;
    */
   public void set(final String key, final long value)
   {
      backing.setProperty(key, String.valueOf(value));
   }

   /**
    * Sets a configuration parameter.
    * 
    * @param key the key.
    * @param value the value;
    */
   public void set(final String key, final float value)
   {
      backing.setProperty(key, String.valueOf(value));
   }

   /**
    * Sets a configuration parameter.
    * 
    * @param key the key.
    * @param value the value;
    */
   public void set(final String key, final double value)
   {
      backing.setProperty(key, String.valueOf(value));
   }

   /**
    * Sets a configuration parameter.
    * 
    * @param key the key.
    * @param value the value;
    */
   public void set(final String key, final boolean value)
   {
      backing.setProperty(key, String.valueOf(value));
   }

   /**
    * Sets a configuration parameter.
    * 
    * @param key the key.
    * @param value the value;
    */
   public void set(final String key, final String value)
   {
      if (value == null)
      {
         unset(key);
      }
      else
      {
         backing.setProperty(key, value.trim());
      }
   }
   
   /**
    * Retrieves a read-only view of this configuration as a Properties map.
    * @return a read-only view of this configuration.
    */
   public Map<Object, Object> getProperties()
   {
      return Collections.unmodifiableMap(backing);
   }

   /**
    * Retrieves the value of the given parameter.
    *  
    * @param key the parameter key.
    * @return the parameter value, or null if not set.
    */
   public String get(final String key)
   {
      String value = backing.getProperty(key);
      return value == null ? null : value.trim();
   }

   /**
    * Retrieves the value of the given parameter.
    *  
    * @param key the parameter key.
    * @param defaultValue the value to return if the parameter is not set.
    * @return the parameter value, or <code>defaultValue</code> if not set.
    */
   public String get(final String key, final String defaultValue)
   {
      String value = backing.getProperty(key, defaultValue);
      return value == null ? null : value.trim();
   }

   /**
    * Retrieves the value of the given parameter.
    *  
    * @param key the parameter key.
    * @param defaultValue the value to return if the parameter is not set or is not valid.
    * @return the parameter value, or <code>defaultValue</code> if not set or not valid.
    */
   public int getInt(final String key, final int defaultValue)
   {
      try
      {
         String s = get(key);
         return s == null ? defaultValue : Integer.parseInt(s);
      }
      catch (Exception e)
      {
      }

      return defaultValue;
   }

   /**
    * Retrieves the value of the given parameter.
    *  
    * @param key the parameter key.
    * @param defaultValue the value to return if the parameter is not set or is not valid.
    * @return the parameter value, or <code>defaultValue</code> if not set or not valid.
    */
   public long getLong(final String key, final long defaultValue)
   {
      try
      {
         String s = get(key);
         return s == null ? defaultValue : Long.parseLong(s);
      }
      catch (Exception e)
      {
      }

      return defaultValue;
   }

   /**
    * Retrieves the value of the given parameter.
    *  
    * @param key the parameter key.
    * @param defaultValue the value to return if the parameter is not set or is not valid.
    * @return the parameter value, or <code>defaultValue</code> if not set or not valid.
    */
   public float getFloat(final String key, final float defaultValue)
   {
      try
      {
         String s = get(key);
         return s == null ? defaultValue : Float.parseFloat(s);
      }
      catch (Exception e)
      {
      }

      return defaultValue;
   }

   /**
    * Retrieves the value of the given parameter.
    *  
    * @param key the parameter key.
    * @param defaultValue the value to return if the parameter is not set or is not valid.
    * @return the parameter value, or <code>defaultValue</code> if not set or not valid.
    */
   public double getDouble(final String key, final double defaultValue)
   {
      try
      {
         String s = get(key);
         return s == null ? defaultValue : Double.parseDouble(s);
      }
      catch (Exception e)
      {
      }

      return defaultValue;
   }

   public boolean getBoolean(final String key)
   {
      return "true".equals(get(key));
   }

   public boolean getBoolean(final String key, final boolean defaultValue)
   {
      String s = get(key);
      return s == null ? defaultValue : "true".equals(s);
   }

   /**
    * Retrieves a Config instance based on the contents of file with the given name.
    * If the file does not exist, an empty Config instance will be returned.
    * 
    * @param fileName the file name to read from.
    * @param autoReload true to automatically reload the Configuration if the file changes.
    * @return the new Config instance.
    */
   public static Config getInstance(final String fileName, final boolean autoReload)
   {
      final Config config = new Config();
      config.read(fileName);
      
      if (autoReload)
      {
         Thread reloadThread = new Thread() 
         {
            @Override
            public void run()
            {
               File file = new File(fileName);
               long lastModified = file.lastModified();
               
               for (;;)
               {
                  if (file.lastModified() > lastModified)
                  {
                     lastModified = file.lastModified();
                     config.read(fileName);
                  }                  
                  
                  try
                  {
                     Thread.sleep(1000);
                  }
                  catch (InterruptedException e)
                  {
                     return;
                  }
               }
            }
         };
         
         reloadThread.setName("configMonitor");
         reloadThread.setDaemon(true);
         reloadThread.start();
      }
      
      return config;
   }

   /**
    * Attempts to read the configuration.
    * @param filename the name of the file to read from.
    */
   private synchronized void read(final String filename)
   {
      FileInputStream in = null;
      
      try
      {
         Properties newBacking = new Properties();
         in = new FileInputStream(filename);         
         newBacking.load(in);
         backing = newBacking;
      }
      catch (Exception e)
      {
      }
      finally
      {
         Util.safeClose(in);
      }
   }

   /**
    * Writes the configuration to the given file.
    * @param filename the file to write to.
    * @return true if the config was written, false otherwise.
    */
   public synchronized boolean write(final String filename)
   {
      FileOutputStream out = null;
      
      try
      {
         out = new FileOutputStream(filename);
         backing.store(out, "");
         out.flush();
      }
      catch (IOException e)
      {
         return false;
      }
      finally
      {
         Util.safeClose(out);
      }

      return true;
   }
}
