package com.aleax.voip.util;

import java.text.NumberFormat;

import javax.swing.JFormattedTextField;
import javax.swing.JTextField;
import javax.swing.text.NumberFormatter;

/**
 * IntegerTextField - extends JTextfield to only accept integers.
 *
 * @version 1.0 14 September 2000
 * @author Yiannis Paschalidis
 */
public class IntegerTextField extends JFormattedTextField
{
   /** The minimum value to allow. */
   private final int minValue;
   
   /** The maximum value to allow. */
   private final int maxValue;
   
   /** 
    * Creates an IntegerTextField that accepts any integer value. 
    */
   public IntegerTextField()
   {
      this(Integer.MIN_VALUE, Integer.MAX_VALUE);
   }

   /**
    * Creates an IntegerTextField that only integer values between
    * <code>minValue</code> and <code>maxValue</code> inclusive.
    * 
    * @param minValue the minimum value to allow.
    * @param maxValue the maximum value to allow.
    */
   public IntegerTextField(final int minValue, final int maxValue)
   {
      this.minValue = minValue;
      this.maxValue = maxValue;
      setColumns(Math.max(String.valueOf(minValue).length(), String.valueOf(maxValue).length()));

      if (minValue > maxValue)
      {
         throw new IllegalArgumentException("minValue>maxValue");
      }

      setText(String.valueOf(minValue));
      setHorizontalAlignment(JTextField.RIGHT);
      
      NumberFormat intFormat = NumberFormat.getIntegerInstance();
      intFormat.setGroupingUsed(false);
      
      NumberFormatter numberFormatter = new NumberFormatter(intFormat);
      numberFormatter.setValueClass(Integer.class);
      numberFormatter.setAllowsInvalid(false);
      numberFormatter.setMinimum(minValue);
      numberFormatter.setMaximum(maxValue);

      setFormatter(numberFormatter);      
   }
   
   /** @return the value entered by the user. */
   public int getIntValue()
   {
      try
      {
         return Integer.parseInt(getText());
      }
      catch (NumberFormatException e)
      {
         // Should never happen
         return 0;
      }
   }
   
   /**
    * Sets the field value.
    * @param value the value to set.
    */
   public void setIntValue(final int value)
   {
      setText(String.valueOf(Math.max(Math.min(value, maxValue), minValue)));
   }
}
