package com.aleax.voip.util;

import java.awt.Toolkit;

import javax.swing.JTextField;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;

/**
 * LengthLimitedTextField - extends JTextfield to only accept a certain number of characters.
 *
 * @author Yiannis Paschalidis
 * @version 1.0 14 September 2000
 */
public class LengthLimitedTextField extends JTextField
{
   /**
    * Creates a LengthLimitedTextField with the length and visible length set the same.
    * 
    * @param maxLength the maximum number of characters to allow.
    */
   public LengthLimitedTextField(final int maxLength)
   {
      this(maxLength, maxLength);
   }

   /** 
    * Creates a LengthLimitedTextField with the given length and visible length.
    * 
    * @param maxLength the maximum number of characters to allow.
    * @param visibleLength the visible length.
    */
   public LengthLimitedTextField(final int maxLength, final int visibleLength)
   {
      super(visibleLength);

      if (maxLength > 0)
      {
         setDocument(new LengthLimitedDocument(maxLength));
      }
   }

   /**
    * Limits the amount of text the document can hold.
    */
   private static final class LengthLimitedDocument extends PlainDocument
   {
      /** The maximum length, in characters. */
      private final int maxLength;

      /**
       * Creates a LengthLimitedDocument.
       * @param maxLength the maximum length, in characters.
       */
      LengthLimitedDocument(final int maxLength)
      {
         this.maxLength = maxLength;
      }

      /** {@inheritDoc} */
      @Override
      public void insertString(final int offs, String str, final AttributeSet a)
            throws BadLocationException
      {
         // Fix for lf being pasted in!
         str = str.replace('\n', ' ');

         String text = getText(0, getLength());

         if (text.length() + str.length() > maxLength)
         {
            str = str.substring(0, maxLength - text.length());
            Toolkit.getDefaultToolkit().beep();
         }

         super.insertString(offs, str, a);
      }
   }
}
