package com.aleax.voip.util;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.image.ColorModel;
import java.awt.image.MemoryImageSource;
import java.awt.image.PixelGrabber;
import java.io.BufferedReader;
import java.io.Closeable;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.URL;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.JPanel;

/**
 * General utility methods.
 *
 * @author Yiannis Paschalidis
 */
public final class Util
{
   /** The logger instance for this class. */
   private static final Logger log = LogUtil.get(Util.class);
   
   /** Prevent instantiation of this utility class. */
   private Util()
   {
   }
   
   /**
    * @return the local (internal-facing) IP address.
    */
   public static String getIntIp()
   {
      try
      {
         return InetAddress.getLocalHost().getHostAddress().toString();
      }
      catch (Exception e)
      {
         log.log(Level.WARNING, "Failed to determine local IP address", e);
         return null;
      }
   }

   /**
    * @return the external (internet-facing) IP address.
    */
   public static String getExtIp()
   {
      String[] services =
      {
         "http://checkip.amazonaws.com",
         "http://icanhazip.com/",
         "http://ifconfig.me/ip"
      };

      BufferedReader rdr = null;

      for (String service : services)
      {
         try
         {
            rdr = new BufferedReader(new InputStreamReader(new URL(service).openStream()));
            String ip = rdr.readLine();

            if (ip != null)
            {
               InetAddress.getByName(ip); // basic sanity check
               return ip;
            }
         }
         catch (Exception e)
         {
            // Continue with next service.
            log.log(Level.WARNING, "IP lookup service " + service + " failed.", e); 
         }
         finally
         {
            safeClose(rdr);
         }
      }

      return "NO INTERNET";
   }

   /**
    * Safely closes the given Closeable, ignoring any errors.
    * @param closeable the Closeable to close.
    */
   public static void safeClose(final Closeable closeable)
   {
      if (closeable != null)
      {
         try
         {
            closeable.close();
         }
         catch (IOException e)
         {
            log.log(Level.FINER, "Close failed", e);
         }
      }
   }
   
   /**
    * Safely closes the given AutoCloseable, ignoring any errors.
    * @param closeable the AutoCloseable to close.
    */
   public static void safeClose(final AutoCloseable closeable)
   {
      if (closeable != null)
      {
         try
         {
            closeable.close();
         }
         catch (Exception e)
         {
            log.log(Level.FINER, "Close failed", e);
         }
      }
   }
   
   /**
    * Loads an image resource from the classpath. 
    * @param name the name of the image to load.
    * @return the loaded image.
    */
   public static Image getImage(final String name)
   {
      Image image = Toolkit.getDefaultToolkit().createImage(Util.class.getResource(name));
      waitForImage(image);
      return image;
   }

   /**
    * Alters the opacity of the given image.
    * 
    * @param img the image to modify.
    * @param alpha the new alpha channel.
    * @param useExistingAlpha true to use the existing alpha channel, false to modify all pixels.
    * @return the modified image.
    */
   public static Image changeAlpha(final Image img, int alpha, final boolean useExistingAlpha)
   {
      int w = img.getWidth(null);
      int h = img.getHeight(null);

      int[] p = new int[w * h];
      PixelGrabber pg = new PixelGrabber(img, 0, 0, w, h, p, 0, w);

      try
      {
         pg.grabPixels();
      }
      catch (InterruptedException exception)
      {
         return null;
      }

      alpha = (alpha & 0xff) << 24;

      if (useExistingAlpha)
      {
         for (int i = p.length - 1 ; i >= 0 ; i--)
         {
            if ((p[i] & 0xFF000000) != 0)
            {
               p[i] = (p[i] & 0x00FFFFFF) | alpha;
            }
         }
      }
      else
      {
         for (int i = p.length - 1 ; i >= 0 ; i--)
         {
            p[i] = (p[i] & 0x00FFFFFF) | alpha;
         }
      }

      MemoryImageSource mis =
            new MemoryImageSource(w, h, ColorModel.getRGBdefault(), p, 0, w);

      return Toolkit.getDefaultToolkit().createImage(mis);     
   }
   
   /** 
    * Waits for an image to load.
    * @param img the image to wait for.
    */
   public static void waitForImage(final Image img)
   {
      MediaTracker mt = new MediaTracker(new JPanel());
      mt.addImage(img, 1);

      try
      {
         mt.waitForAll();
      }
      catch (InterruptedException e)
      {
      }
   }

   /**
    * Centers a window.
    * @param window the window to center.
    * @param parent the parent component to center the window on, or null for top-level.
    */
   public static void centerWindow(final Window window, Component parent)
   {
      // If we're not given a top-level container, find it...
      parent = getParentWindow(parent);

      Dimension parentSize;
      Point offset = new Point();

      if (parent == null)
      {
         parentSize = Toolkit.getDefaultToolkit().getScreenSize();
      }
      else
      {
         parentSize = parent.getSize();
         offset = parent.getLocation();
      }

      window.setLocation(offset.x + (parentSize.width - window.getWidth()) / 2, 
                         offset.y + (parentSize.height - window.getHeight()) / 2);
   }
   
   /** @return the parent Window of the given component, null if there isn't one */
   public static Window getParentWindow(Component c)
   {
      while (c != null && !(c instanceof Window))
      {
         c = c.getParent();
      }

      return (Window)c;
   }
   
   /**
    * Null-safe version of equals.
    * 
    * @param obj1 the first object to compare
    * @param obj2 the second object to compare
    * 
    * @return true if o1 == o2, or o1.equals(o2)
    */
   public static boolean equals(final Object obj1, final Object obj2)
   {
      if (obj1 == obj2)
      {
         return true;
      }
      else if (obj1 != null)
      {
         return obj1.equals(obj2);
      }
      
      return false;
   }
   
   /**
    * Null-safe version of compare. Nulls are considered less than non-nulls.
    * 
    * @param obj1 the first object to compare
    * @param obj2 the second object to compare
    * @return -ve, zero or +ve depending on whether o2 is less than, equal or greater than o1.
    * @param <T> the type of object to compare.
    */
   public static <T extends Comparable<T>> int compare(final T obj1, final T obj2)
   {
      if (obj1 == null)
      {
         return obj2 == null ? 0 : -1;
      }
      else if (obj2 == null)
      {
         return 1;
      }
      else
      {
         return obj1.compareTo(obj2);
      }
   }

   /** 
    * Checks whether the given string is empty.
    * A String is considered empty if it is null or
    * only contains whitespace/control characters.
    * 
    * @param string the String to check.
    * @return true if the given String is empty.
    */
   public static boolean empty(final String string)
   {
      if (string != null)
      {
         final int len = string.length();

         for (int i = 0 ; i < len ; i++)
         {
            if (string.charAt(i) > 32)
            {
               return false;
            }
         }
      }

      return true;
   }
}
