package com.aleax.voip.util;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.JPanel;

/**
 * An extension of JPanel which adds more functionality.
 *
 * @author Yiannis Paschalidis
 */
public class XPanel extends JPanel
{
   /** Layout manager. */
   private final GridBagLayout gridBagLayout;

   /** Temporary object used to add components to the layout. */
   private final GridBagConstraints gridBagConstraints;

   /** Creates an XPanel. */
   public XPanel()
   {
      gridBagLayout = new GridBagLayout();
      gridBagConstraints = new GridBagConstraints();
      setLayout(gridBagLayout);
   }

   /**
    * Adds a component to the XPanel at the specified gridbag coordinates.
    *
    * @param c the component to add
    * @param gridx the x grid coordinate
    * @param gridy the y grid coordinate
    */
   public void add(final Component c, final int gridx, final int gridy)
   {
      add(c, gridx, gridy, GridBagConstraints.NONE, 1, 1, GridBagConstraints.CENTER);
   }

   /**
    * Adds a component to the XPanel at the specified gridbag coordinates,
    * weighted to expand horizontally.
    *
    * @param c the component to add
    * @param gridx the x grid coordinate
    * @param gridy the y grid coordinate
    */
   public void addFillX(final Component c, final int gridx, final int gridy)
   {
      add(c, gridx, gridy, GridBagConstraints.HORIZONTAL, 1, 1, GridBagConstraints.CENTER);
   }

   /**
    * Adds a component to the XPanel with the specified gridbag parameters,
    * weighted to expand horizontally.
    *
    * @param c the component to add
    * @param gridx the x grid coordinate
    * @param gridy the y grid coordinate
    * @param gridwidth the number of horizontal grid cells the component will occupy
    * @param gridheight the number of vertical grid cells the component will occupy
    */
   public void addFillX(final Component c, final int gridx, final int gridy, final int gridwidth,
         final int gridheight)
   {
      add(c, gridx, gridy, GridBagConstraints.HORIZONTAL, gridwidth, gridheight, GridBagConstraints.CENTER);
   }

   /**
    * Adds a component to the XPanel at the specified gridbag coordinates,
    * weighted to expand vertically.
    *
    * @param c the component to add
    * @param gridx the x grid coordinate
    * @param gridy the y grid coordinate
    */
   public void addFillY(final Component c, final int gridx, final int gridy)
   {
      add(c, gridx, gridy, GridBagConstraints.VERTICAL, 1, 1, GridBagConstraints.CENTER);
   }

   /**
    * Adds a component to the XPanel with the specified gridbag parameters,
    * weighted to expand vertically.
    *
    * @param c the component to add
    * @param gridx the x grid coordinate
    * @param gridy the y grid coordinate
    * @param gridwidth the number of horizontal grid cells the component will occupy
    * @param gridheight the number of vertical grid cells the component will occupy
    */
   public void addFillY(final Component c, final int gridx, final int gridy, final int gridwidth,
         final int gridheight)
   {
      add(c, gridx, gridy, GridBagConstraints.VERTICAL, gridwidth, gridheight, GridBagConstraints.CENTER);
   }

   /**
    * Adds a component to the XPanel at the specified gridbag coordinates,
    * weighted to expand horizontally and vertically.
    *
    * @param c the component to add
    * @param gridx the x grid coordinate
    * @param gridy the y grid coordinate
    */
   public void addFillXY(final Component c, final int gridx, final int gridy)
   {
      add(c, gridx, gridy, GridBagConstraints.BOTH, 1, 1, GridBagConstraints.CENTER);
   }

   /**
    * Adds a component to the XPanel with the specified gridbag parameters,
    * weighted to expand horizontally and vertically.
    *
    * @param c the component to add
    * @param gridx the x grid coordinate
    * @param gridy the y grid coordinate
    * @param gridwidth the number of horizontal grid cells the component will occupy
    * @param gridheight the number of vertical grid cells the component will occupy
    */
   public void addFillXY(final Component c, final int gridx, final int gridy, final int gridwidth,
         final int gridheight)
   {
      add(c, gridx, gridy, GridBagConstraints.BOTH, gridwidth, gridheight, GridBagConstraints.CENTER);
   }

   /**
    * Adds a component to the XPanel with the specified gridbag parameters.
    *
    * @param c the component to add
    * @param gridx the x grid coordinate
    * @param gridy the y grid coordinate
    * @param anchor the position of the component within the grid cell. See {@link GridBagConstraints#anchor}.
    */
   public void add(final Component c, final int gridx, final int gridy, final int anchor)
   {
      add(c, gridx, gridy, GridBagConstraints.NONE, 1, 1, anchor);
   }

	/** 
	 * Adds a component to the XPanel with the specified gridbag parameters.
	 *
	 * @param c the component to add
	 * @param gridx the x grid coordinate
	 * @param gridy the y grid coordinate
	 * @param gridwidth the number of horizontal grid cells the component will occupy
	 * @param gridheight the number of vertical grid cells the component will occupy
	 */
   public void add(final Component c, final int gridx, final int gridy, final int gridwidth, final int gridheight)
   {
      add(c, gridx, gridy, GridBagConstraints.NONE, gridwidth, gridheight, GridBagConstraints.CENTER);
   }

	/** 
	 * Adds a component to the XPanel with the specified gridbag parameters.
	 *
	 * @param c the component to add
	 * @param gridx the x grid coordinate
	 * @param gridy the y grid coordinate
	 * @param fill the fill Mode of the component. See {@link GridBagConstraints#fill}.
	 * @param gridwidth the number of horizontal grid cells the component will occupy
	 * @param gridheight the number of vertical grid cells the component will occupy
	 */
   public void add(final Component c, final int gridx, final int gridy, final int fill, final int gridwidth, final int gridheight)
   {
      add(c, gridx, gridy, fill, gridwidth, gridheight, GridBagConstraints.CENTER);
   }

	/** 
	 * Adds a component to the XPanel with the specified gridbag parameters.
	 *
	 * @param c the component to add
	 * @param gridx the x grid coordinate
	 * @param gridy the y grid coordinate
	 * @param fill the fill Mode of the component. See {@link GridBagConstraints#fill}.
	 * @param gridwidth the number of horizontal grid cells the component will occupy
	 * @param gridheight the number of vertical grid cells the component will occupy
	 * @param anchor the position of the component within the grid cell. See {@link GridBagConstraints#anchor}.
	 */
   public void add(final Component c, final int gridx, final int gridy, final int fill,
         final int gridwidth, final int gridheight, final int anchor)
   {
      if (fill == GridBagConstraints.HORIZONTAL)
      {
         gridBagConstraints.weightx = 1.0;
         gridBagConstraints.weighty = 0.0;
      }
      else if (fill == GridBagConstraints.VERTICAL)
      {
         gridBagConstraints.weightx = 0.0;
         gridBagConstraints.weighty = 1.0;
      }
      else if (fill == GridBagConstraints.BOTH)
      {
         gridBagConstraints.weightx = 1.0;
         gridBagConstraints.weighty = 1.0;
      }
      else
      {
         gridBagConstraints.weightx = 0;
         gridBagConstraints.weighty = 0;
      }

      gridBagConstraints.gridx = gridx;
      gridBagConstraints.gridy = gridy;

      gridBagConstraints.gridwidth = gridwidth;
      gridBagConstraints.gridheight = gridheight;

      gridBagConstraints.fill = fill;
      gridBagConstraints.anchor = anchor;

      gridBagLayout.setConstraints(c, gridBagConstraints);
      super.add(c);
   }
}
