package com.aleax.voip.util;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;

import javax.swing.JSplitPane;

/**
 * An extension of JSplitPane which behaves more sanely when it is resized.
 * 
 * @author Yiannis Paschalidis
 */
public class XSplitPane extends JSplitPane
{
   private boolean ignoreDividerLocationChange = false;
   private boolean painted;
   private double defaultDividerLocation;

   /**
    * Creates a new XSplitPane with the given orientation.
    *
    * @param orientation  <code>JSplitPane.HORIZONTAL_SPLIT</code> or
    *                      <code>JSplitPane.VERTICAL_SPLIT</code>
    */
   public XSplitPane(final int orientation)
   {
      super(orientation);
   }

   /**
    * Creates a new XSplitPane with the given orientation and contents.
    *
    * @param orientation <code>JSplitPane.HORIZONTAL_SPLIT</code> or
    *                     <code>JSplitPane.VERTICAL_SPLIT</code>
    * @param first the top/left component.
    * @param second the botton/right component.
    */
   public XSplitPane(final int splitType, final Component first, final Component second)
   {
      super(splitType, first, second);
      setContinuousLayout(true);
   }

   /** {@inheritDoc} */
   @Override
   public void setDividerLocation(final double proportionalLocation)
   {
      this.defaultDividerLocation = proportionalLocation;
      
      ignoreDividerLocationChange = true;
      super.setDividerLocation(proportionalLocation);
      ignoreDividerLocationChange = false;
   }
   
   /** {@inheritDoc} */
   @Override
   public void setDividerLocation(final int location)
   {
      super.setDividerLocation(location);
      
      if (!ignoreDividerLocationChange)
      {
         defaultDividerLocation = location / (double) getHeight(); 
      }
   }
   
   /** {@inheritDoc} */
   @Override
   public void doLayout()
   {
      ignoreDividerLocationChange = true;
      super.doLayout();
      repositionDivider();
      ignoreDividerLocationChange = false;
   }

   /** {@inheritDoc} */
   @Override
   public void paint(final Graphics g)
   {
      ignoreDividerLocationChange = true;
      super.paint(g);

      if (!painted)
      {
         painted = true;
         repositionDivider();
      }
      
      ignoreDividerLocationChange = false;
   }

   /** {@inheritDoc} */
   @Override
   public void setSize(final Dimension size)
   {
      ignoreDividerLocationChange = true;
      super.setSize(size);
      repositionDivider();
      ignoreDividerLocationChange = false;
   }
   
   /** {@inheritDoc} */
   @Override
   public void setSize(final int width, final int height)
   {
      ignoreDividerLocationChange = true;
      super.setSize(width, height);
      repositionDivider();
      ignoreDividerLocationChange = false;
   }
   
   /** 
    * Repositions the divider based on the proportional location. 
    */
   private void repositionDivider()
   {
      setDividerLocation((int) (defaultDividerLocation * getHeight()));
   }
}
