package com.aleax.voip.util;

import java.awt.Component;
import java.awt.FontMetrics;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.border.EmptyBorder;

/**
 * Tab pane extension which supports closing tabs.
 * 
 * @author Yiannis Paschalidis
 */
public class XTabbedPane extends JTabbedPane
{
   /** Listeners to notify when a tab is closed. */
   private final List<TabCloseListener> listeners = new ArrayList<TabCloseListener>();
   
   /** {@inheritDoc} */
   @Override
   public void addTab(final String title, final Component component)
   {
      super.addTab(title, component);
      setTabComponentAt(indexOfComponent(component), getTabLabel(component, title));
   }
   
   /**
    * Creates a tab label.
    * 
    * @param component the component which was added.
    * @param title the tab title.
    * @return the new tab label.
    */
   private JPanel getTabLabel(final Component component, final String title)
   {
      XPanel panel = new XPanel();
      panel.setBorder(new EmptyBorder(5, 0, 0, 0));
      panel.setOpaque(false);

      FontMetrics fm = Toolkit.getDefaultToolkit().getFontMetrics(new JButton().getFont());
      int size = Math.max(fm.stringWidth("X"), fm.getHeight());      
      Image image = Util.getImage("/images/close.png");
      image = image.getScaledInstance(size, size, Image.SCALE_SMOOTH);
      JButton closeButton = new JButton(null, new ImageIcon(image));
      closeButton.setBorder(new EmptyBorder(0, 5, 0, 0));      
      closeButton.setContentAreaFilled(false);
      closeButton.setOpaque(false);
      closeButton.setToolTipText("Close");
      panel.add(closeButton, 1, 0);
      
      JLabel label = new JLabel(title);
      panel.addFillX(label, 0, 0);

      closeButton.addActionListener(new ActionListener()
      {
         @Override
         public void actionPerformed(final ActionEvent e)
         {
            remove(component);
            
            for (TabCloseListener listener : listeners)
            {
               listener.tabClosed(component);
            }
         }
      });
      
      return panel;
   }

   /**
    * Adds a listener to receive tab close events.
    * @param listener the listener to add.
    */
   public void addListener(final TabCloseListener listener)
   {
      listeners.add(listener);
   }
   
   /**
    * Removes a listener from receiving close events.
    * @param listener the listener to add.
    */
   public void removeListener(final TabCloseListener listener)
   {
      listeners.remove(listener);
   }

   /**
    * TabCloseListeners can receive notifications when a tab is closed.
    * 
    * @see XTabbedPane#addListener(TabCloseListener)
    */
   public interface TabCloseListener
   {
      /**
       * Called when a tab is closed.
       * @param tab the component which was in the body of the tab.
       */
      void tabClosed(Component tab);
   }
}
